@@ -1,7 +1,7 @@
 #ifndef __GDICT_ABOUT_H_
 #define __GDICT_ABOUT_H_
 
-/* $Id: gdict-about.h,v 1.2 2002/06/01 21:32:46 gman Exp $ */
+/* $Id: gdict-about.h,v 1.4 2004/11/03 12:18:42 pborelli Exp $ */
 
 #include "gtk/gtkwindow.h"
 /*
@@ -15,7 +15,8 @@
  *
  */
 
-extern void gdict_about (GtkWindow *parent);
+GtkWidget *gdict_about_new (void);
+void gdict_about (GtkWindow *parent);
 
 #endif /* __GDICT_ABOUT_H_ */
 
