@@ -1,4 +1,4 @@
-/* $Id: gdict-defbox.c,v 1.23 2004/05/11 23:23:28 fherrera Exp $ */
+/* $Id: gdict-defbox.c,v 1.34 2004/12/16 15:52:42 vnoel Exp $ */
 
 /*
  *  Mike Hughes <mfh@psilord.com>
@@ -16,22 +16,11 @@
 #  include <config.h>
 #endif
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <assert.h>
-#include <string.h>
-#include <ctype.h>
-
-#define GTK_ENABLE_BROKEN
-#include <gnome.h>
-
+#include <glib/gi18n.h>
 #include "gdict-defbox.h"
 #include "gdict-app.h"
-#include "gdict-pref.h"
-
-#ifdef HAVE_GNOME_PRINT
-#  include <math.h>
-#endif /* HAVE_GNOME_PRINT */
+#include <libgnomeprint/gnome-print-pango.h>
+#include <libgnomeprintui/gnome-print-job-preview.h>
 
 enum {
     WORD_LOOKUP_START_SIGNAL,
@@ -42,6 +31,10 @@ enum {
     LAST_SIGNAL
 };
 
+#define LEFT_MARGIN 72.
+#define RIGHT_MARGIN 72.
+#define TOP_MARGIN 72.
+
 static gint gdict_defbox_signals[LAST_SIGNAL] = { 0, 0, 0, 0 };
 
 static void gdict_defbox_init (GDictDefbox *defbox);
@@ -59,19 +52,9 @@ static gboolean is_number (gchar *str, int len);
 static gboolean is_part   (gchar *str, int len);
 static void defbox_add    (GDictDefbox *defbox, gchar *def);
 
-#ifdef HAVE_GNOME_PRINT
-static GnomeTextLine *get_line (const char *fontlist, const char *string, 
-                                double size);
-#endif
-
-/* gdict_defbox_get_type
- *
- * Register the GDictDefbox type with Gtk's type system if necessary and
- * return the type identifier code
- */
-
 GType 
-gdict_defbox_get_type (void) {
+gdict_defbox_get_type (void)
+{
     static GType gdict_defbox_type = 0;
     
     if (!gdict_defbox_type) {
@@ -95,81 +78,89 @@ gdict_defbox_get_type (void) {
     return gdict_defbox_type;
 }
 
-/* gdict_defbox_init
- *
- * Initialises an instance of a GDictDefbox object
- */
-
 static void 
-gdict_defbox_init (GDictDefbox *defbox) {
+gdict_defbox_init (GDictDefbox *defbox)
+{
     defbox->context = NULL;
     defbox->def_cmd = NULL;
 }
 
-/* gdict_defbox_class_init
- *
- * Initialises a structure describing the GDictDefbox class; sets up signals
- * for defbox events in the Gtk signal management system
- */
-
 static void 
-gdict_defbox_class_init (GDictDefboxClass *class) {
-    GtkObjectClass *object_class;
-    
-    object_class = (GtkObjectClass *) class;
+gdict_defbox_class_init (GDictDefboxClass *class)
+{
+    GObjectClass *object_class;
+
+    object_class = G_OBJECT_CLASS (class);
     
     gdict_defbox_signals[WORD_LOOKUP_START_SIGNAL] =
-        gtk_signal_new ("word_lookup_start", GTK_RUN_FIRST, GTK_CLASS_TYPE (object_class),
-                        GTK_SIGNAL_OFFSET (GDictDefboxClass, word_lookup_start),
-                        gtk_signal_default_marshaller, GTK_TYPE_NONE, 0);
-    
+        g_signal_new ("word_lookup_start",
+                      G_TYPE_FROM_CLASS (object_class),
+                      G_SIGNAL_RUN_FIRST,
+                      G_STRUCT_OFFSET (GDictDefboxClass, word_lookup_start),
+                      NULL, NULL,
+                      g_cclosure_marshal_VOID__VOID,
+                      G_TYPE_NONE, 0);
+
     gdict_defbox_signals[WORD_LOOKUP_DONE_SIGNAL] =
-        gtk_signal_new ("word_lookup_done", GTK_RUN_FIRST, GTK_CLASS_TYPE (object_class),
-                        GTK_SIGNAL_OFFSET (GDictDefboxClass, word_lookup_done),
-                        gtk_signal_default_marshaller, GTK_TYPE_NONE, 0);
-    
+        g_signal_new ("word_lookup_done",
+                      G_TYPE_FROM_CLASS (object_class),
+                      G_SIGNAL_RUN_FIRST,
+                      G_STRUCT_OFFSET (GDictDefboxClass, word_lookup_done),
+                      NULL, NULL,
+                      g_cclosure_marshal_VOID__VOID,
+                      G_TYPE_NONE, 0);
+
     gdict_defbox_signals[WORD_NOT_FOUND_SIGNAL] =
-        gtk_signal_new ("word_not_found", GTK_RUN_FIRST, GTK_CLASS_TYPE (object_class),
-                        GTK_SIGNAL_OFFSET (GDictDefboxClass, word_not_found),
-                        gtk_signal_default_marshaller, GTK_TYPE_NONE, 0);
-    
+        g_signal_new ("word_not_found",
+                      G_TYPE_FROM_CLASS (object_class),
+                      G_SIGNAL_RUN_FIRST,
+                      G_STRUCT_OFFSET (GDictDefboxClass, word_not_found),
+                      NULL, NULL,
+                      g_cclosure_marshal_VOID__VOID,
+                      G_TYPE_NONE, 0);
+
     gdict_defbox_signals[SUBSTR_NOT_FOUND_SIGNAL] =
-        gtk_signal_new ("substr_not_found", GTK_RUN_FIRST, GTK_CLASS_TYPE (object_class),
-                        GTK_SIGNAL_OFFSET (GDictDefboxClass, substr_not_found),
-                        gtk_signal_default_marshaller, GTK_TYPE_NONE, 0);
-    
+        g_signal_new ("substr_not_found",
+                      G_TYPE_FROM_CLASS (object_class),
+                      G_SIGNAL_RUN_FIRST,
+                      G_STRUCT_OFFSET (GDictDefboxClass, substr_not_found),
+                      NULL, NULL,
+                      g_cclosure_marshal_VOID__VOID,
+                      G_TYPE_NONE, 0);
+
     gdict_defbox_signals[SOCKET_ERROR_SIGNAL] =
-        gtk_signal_new ("socket_error", GTK_RUN_FIRST, GTK_CLASS_TYPE (object_class),
-                        GTK_SIGNAL_OFFSET (GDictDefboxClass, socket_error),
-                        gtk_marshal_VOID__STRING, GTK_TYPE_NONE, 1,
-                        GTK_TYPE_STRING);
- /*   
-    gtk_object_class_add_signals (object_class, gdict_defbox_signals,
-                                  LAST_SIGNAL);
- */  
+        g_signal_new ("socket_error",
+                      G_TYPE_FROM_CLASS (object_class),
+                      G_SIGNAL_RUN_FIRST,
+                      G_STRUCT_OFFSET (GDictDefboxClass, socket_error),
+                      NULL, NULL,
+                      g_cclosure_marshal_VOID__VOID,
+                      G_TYPE_NONE,
+                      1,
+                      G_TYPE_STRING);
+
     class->word_lookup_start = NULL;
     class->word_lookup_done = NULL;
     class->word_not_found = NULL;
     class->substr_not_found = NULL;
 }
 
-/* gdict_defbox_new
- *
- * Creates a new GDictDefbox object
- */
-
 GtkWidget *
-gdict_defbox_new (void) {
+gdict_defbox_new (gpointer gdict)
+{
     GDictDefbox *defbox;
-    
-    defbox = GDICT_DEFBOX (gtk_type_new (gdict_defbox_get_type ()));
+
+    defbox = GDICT_DEFBOX (g_object_new (GDICT_TYPE_DEFBOX, NULL));
+    defbox->gdict = gdict;
+
     return GTK_WIDGET (defbox);
 }
 
 /* Setup textview tags
  */
 void
-defbox_setup_tags (GDictDefbox *defbox) {
+defbox_setup_tags (GDictDefbox *defbox)
+{
     GtkTextBuffer *buffer;
 	
     buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (defbox));
@@ -182,16 +173,6 @@ defbox_setup_tags (GDictDefbox *defbox) {
 			        "underline", PANGO_UNDERLINE_SINGLE, NULL);
 }
 
-/* gdict_defbox_destroy
- *
- * Deallocates memory associated with the defbox
- */
-
-void
-gdict_defbox_destroy (GDictDefbox *defbox) {
-    g_free (defbox->database);
-}
-
 /* gdict_defbox_lookup
  *
  * Sends the command to the server to commence looking up the definition
@@ -202,25 +183,27 @@ gdict_defbox_destroy (GDictDefbox *defbox) {
  */
 
 gint
-gdict_defbox_lookup (GDictDefbox *defbox, gchar *text) {
+gdict_defbox_lookup (GDictDefbox *defbox, gchar *text)
+{
+    GDictWindow *gdict = defbox->gdict;
+
     g_return_val_if_fail (defbox != NULL, -1);
-    g_return_val_if_fail (IS_GDICT_DEFBOX (defbox), -1);
+    g_return_val_if_fail (GDICT_IS_DEFBOX (defbox), -1);
     g_return_val_if_fail (text != NULL, -1);
     
     while (isspace (*text)) text++;
     
     if (*text == '\0')
         return 0;
-    
-    gtk_signal_emit (GTK_OBJECT (defbox), 
-                     gdict_defbox_signals[WORD_LOOKUP_START_SIGNAL]);
-    
+
+    g_signal_emit (G_OBJECT (defbox), gdict_defbox_signals[WORD_LOOKUP_START_SIGNAL], 0);
+
     gdict_defbox_clear (defbox);
     
     if (defbox->database)
 	    g_free (defbox->database);
     
-    defbox->database = g_strdup (gdict_pref.database);
+    defbox->database = g_strdup (gdict->pref->database);
     
     defbox->def_cmd = dict_define_command_new (defbox->database, text);
     defbox->def_cmd->error_notify_cb = def_error_cb;
@@ -240,18 +223,18 @@ gdict_defbox_lookup (GDictDefbox *defbox, gchar *text) {
  */
 
 void 
-gdict_defbox_clear (GDictDefbox *defbox) {
+gdict_defbox_clear (GDictDefbox *defbox)
+{
     GtkTextBuffer *buffer;
     GtkTextIter start, end;
-    
+
     g_return_if_fail (defbox != NULL);
-    g_return_if_fail (IS_GDICT_DEFBOX (defbox));
-    
+    g_return_if_fail (GDICT_IS_DEFBOX (defbox));
+
     buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (defbox));
-    gtk_text_buffer_get_start_iter (buffer, &start);
-    gtk_text_buffer_get_end_iter (buffer, &end);
+    gtk_text_buffer_get_bounds (buffer, &start, &end);
     gtk_text_buffer_delete (buffer, &start, &end);
-  
+
     if (defbox->def_cmd) {
         dict_command_destroy (defbox->def_cmd);
         defbox->def_cmd = NULL;
@@ -264,19 +247,20 @@ gdict_defbox_clear (GDictDefbox *defbox) {
  */
 
 gboolean 
-gdict_defbox_find (GDictDefbox *defbox, const gchar *text, gboolean start) {
+gdict_defbox_find (GDictDefbox *defbox, const gchar *text, gboolean start)
+{
     GtkTextBuffer *buffer;
     GtkTextIter start_iter, end_iter, iter;
     GtkTextIter match_start, match_end;
     GtkTextMark *mark = NULL;
 
     g_return_val_if_fail (defbox != NULL, FALSE);
-    g_return_val_if_fail (IS_GDICT_DEFBOX (defbox), FALSE);
+    g_return_val_if_fail (GDICT_IS_DEFBOX (defbox), FALSE);
     
     buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (defbox));
-    
+
     gtk_text_buffer_get_bounds (buffer, &start_iter, &end_iter);
-    	
+
     if (start) 
     	iter = start_iter;
     else {
@@ -306,8 +290,7 @@ gdict_defbox_find (GDictDefbox *defbox, const gchar *text, gboolean start) {
     }
     else
     {
-        gtk_signal_emit (GTK_OBJECT (defbox), 
-                         gdict_defbox_signals[SUBSTR_NOT_FOUND_SIGNAL]);
+        g_signal_emit (G_OBJECT (defbox), gdict_defbox_signals[SUBSTR_NOT_FOUND_SIGNAL], 0);
 
 	return FALSE;
     }
@@ -320,12 +303,14 @@ gdict_defbox_find (GDictDefbox *defbox, const gchar *text, gboolean start) {
  */
 
 void
-gdict_defbox_reset (GDictDefbox *defbox, dict_context_t *context) {
+gdict_defbox_reset (GDictDefbox *defbox, dict_context_t *context)
+{
     gchar *word;
+    GDictWindow *gdict = defbox->gdict;
     
     if (context != defbox->context || 
         defbox->database == NULL ||
-        strcmp (defbox->database, gdict_pref.database))
+        strcmp (defbox->database, gdict->pref->database))
     {
         defbox->context = context;
         
@@ -339,70 +324,83 @@ gdict_defbox_reset (GDictDefbox *defbox, dict_context_t *context) {
     }
 }
 
-#ifdef HAVE_GNOME_PRINT
+void
+gdict_defbox_print_preview (GDictDefbox *defbox)
+{
+	GtkWidget *preview;
+	GDictWindow *gdict = defbox->gdict;
+	preview = gnome_print_job_preview_new (gdict->print_job, _("Definition preview"));
+	gtk_widget_show (preview);
+}
 
 /* gdict_defbox_print
  * 
- * Prints the current definition
+ * prepare the printing job for the definition printing
  */
 
 void 
-gdict_defbox_print (GDictDefbox *defbox) {
+gdict_defbox_prepare_print (GDictDefbox *defbox)
+{
+    GDictWindow *gdict = defbox->gdict;
     GnomePrintContext *pc;
     GList *node;
     dict_res_t *res;
-    int x = 32, y = 700;
+    int nb_node = 0, i=0, width, height;
+    double page_width, page_height;
+    gchar **node_definition;
+    gchar *text;
+    PangoLayout *layout;
+    PangoFontDescription *desc;
 
     g_return_if_fail (defbox != NULL);
-    g_return_if_fail (IS_GDICT_DEFBOX (defbox));
-    g_return_if_fail (gdict_printer != NULL);
+    g_return_if_fail (GDICT_IS_DEFBOX (defbox));
 
     if (defbox->def_cmd == NULL)
-        return;
+	    return;
 
-    pc = gnome_print_context_new (gdict_printer);
+    pc = gnome_print_job_get_context (gdict->print_job);
     g_return_if_fail(pc != NULL);
+    gnome_print_beginpage (pc, "1");
+    gnome_print_job_get_page_size (gdict->print_job, &page_width, &page_height);
+    gnome_print_moveto (pc, 100, page_height - TOP_MARGIN);
 
+    layout = gnome_print_pango_create_layout (pc);
+    pango_layout_set_width (layout, (page_width - LEFT_MARGIN - RIGHT_MARGIN) * PANGO_SCALE);
+    desc = pango_font_description_from_string ("Times 10");
+    pango_layout_set_font_description (layout, desc);
+    pango_font_description_free (desc);
+
+    pango_layout_get_size (layout, &width, &height);
+
+    nb_node = g_list_length (defbox->def_cmd->res_list);
+
+    node_definition = (gchar **) g_new0 (gpointer, nb_node+1);
     for (node = defbox->def_cmd->res_list; node; node = g_list_next (node)) {
-        gchar *linep, line[1024];  /* FIXME */
-        GnomeTextLine *textline;
-        
-        res = (dict_res_t *) node->data;
-        linep = res->desc;
-
-        while (linep && *linep) {
-            gchar *endp = strchr(linep, '\n') - 1;
-
-            bzero(line, sizeof(line));
-            strncpy(line, linep, endp - linep);
-            textline = get_line("Times", line, 12.0);
-            gnome_print_moveto(pc, x, y);
-            gnome_print_textline(pc, textline);
-            gnome_text_line_free(textline);
-            y -= 14;
-            linep = endp + 2;
-        }
-        textline = get_line("Times", "", 12.0);
-        gnome_print_moveto(pc, x, y);
-        gnome_print_textline(pc, textline);
-        gnome_text_line_free(textline);
-        y -= 14;
+	    res = (dict_res_t *) node->data;
+	    node_definition[i++] = res->desc;
     }
+    node_definition[i] = NULL;
 
-    gnome_print_context_close(pc);
+    text = g_strjoinv ("\n", node_definition);
+
+    pango_layout_set_text (layout, text, -1);
+    gnome_print_pango_layout (pc, layout);
+	    
+    gnome_print_showpage (pc);
+
+    g_object_unref (G_OBJECT (pc));
 }
 
-#endif /* HAVE_GNOME_PRINT */
-
 /* gdict_defbox_get_word
  *
  * Returns the word defined in the defbox, if any
  */
 
 gchar *
-gdict_defbox_get_word (GDictDefbox *defbox) {
+gdict_defbox_get_word (GDictDefbox *defbox)
+{
     g_return_val_if_fail (defbox != NULL, NULL);
-    g_return_val_if_fail (IS_GDICT_DEFBOX (defbox), NULL);
+    g_return_val_if_fail (GDICT_IS_DEFBOX (defbox), NULL);
     
     return defbox->def_cmd ? defbox->def_cmd->search_term : NULL;
 }
@@ -413,7 +411,8 @@ gdict_defbox_get_word (GDictDefbox *defbox) {
  */
 
 static gboolean 
-is_xref (gchar *str, int len) {
+is_xref (gchar *str, int len)
+{
     gint i;
     for (i = 0;  i < len;  i++)
         if (!isupper(str[i]) && !ispunct(str[i]))
@@ -427,7 +426,8 @@ is_xref (gchar *str, int len) {
  */
 
 static gboolean 
-is_number (gchar *str, int len) {
+is_number (gchar *str, int len)
+{
     gint i;
 
     if (str[len-1] != '.')
@@ -446,7 +446,8 @@ is_number (gchar *str, int len) {
  */
 
 static gboolean 
-is_part (gchar *str, int len) {
+is_part (gchar *str, int len)
+{
     gchar buf[3];
 
     if ((len < 1) || (len > 2))
@@ -465,8 +466,8 @@ is_part (gchar *str, int len) {
 
 static void
 insert_text_with_tags (GtkTextBuffer *buffer, GtkTextIter *iter, gchar *p, gint len,
-		       gboolean bold, gboolean italic, gboolean underline) {
-	
+		       gboolean bold, gboolean italic, gboolean underline)
+{
 	GtkTextMark *mark;
 	GtkTextIter start;
 	
@@ -474,6 +475,7 @@ insert_text_with_tags (GtkTextBuffer *buffer, GtkTextIter *iter, gchar *p, gint
 	gtk_text_buffer_insert (buffer, iter, p, len);
 	gtk_text_buffer_get_iter_at_mark (buffer, &start, mark);
 	gtk_text_buffer_delete_mark (buffer, mark);
+
 	if (bold)
 		gtk_text_buffer_apply_tag_by_name (buffer, "bold", &start, iter);
 	if (italic)
@@ -503,9 +505,11 @@ insert_link (GtkTextBuffer *buffer, GtkTextIter *iter, gchar *text, gint len)
  */
 
 static void 
-defbox_add (GDictDefbox *defbox, gchar *def) {
+defbox_add (GDictDefbox *defbox, gchar *def)
+{
     GtkTextBuffer *buffer;
     GtkTextIter iter;
+    GDictWindow *gdict = defbox->gdict;
     gchar *p, *q, *text;
     gint len;
     GdkFont *font;
@@ -518,8 +522,8 @@ defbox_add (GDictDefbox *defbox, gchar *def) {
     
     gtk_text_buffer_insert (buffer, &iter, "\n", -1);
          
-    font = gdict_pref.typefaces[TYPEFACE_HEADWORD].font;
-    color = gdict_pref.typefaces[TYPEFACE_HEADWORD].color;
+    font = gdict->pref->typefaces[TYPEFACE_HEADWORD].font;
+    color = gdict->pref->typefaces[TYPEFACE_HEADWORD].color;
 
     p = defbox->def_cmd->db_full_name;
     len = strlen(p);
@@ -537,13 +541,13 @@ defbox_add (GDictDefbox *defbox, gchar *def) {
         /* Handle word token */
         insert_text_with_tags (buffer, &iter, p, len, bold, italic, underline);
         
-        if ((font == gdict_pref.typefaces[TYPEFACE_HEADWORD].font) || \
-            (font == gdict_pref.typefaces[TYPEFACE_NUMBER].font) || \
-            (font == gdict_pref.typefaces[TYPEFACE_PART].font) || \
-            (font == gdict_pref.typefaces[TYPEFACE_XREF].font))
+        if ((font == gdict->pref->typefaces[TYPEFACE_HEADWORD].font) || \
+            (font == gdict->pref->typefaces[TYPEFACE_NUMBER].font) || \
+            (font == gdict->pref->typefaces[TYPEFACE_PART].font) || \
+            (font == gdict->pref->typefaces[TYPEFACE_XREF].font))
         {
-            font = gdict_pref.typefaces[TYPEFACE_BODY].font;
-            color = gdict_pref.typefaces[TYPEFACE_BODY].color;
+            font = gdict->pref->typefaces[TYPEFACE_BODY].font;
+            color = gdict->pref->typefaces[TYPEFACE_BODY].color;
             bold = FALSE;
             italic = FALSE;
             underline = FALSE;
@@ -564,35 +568,35 @@ defbox_add (GDictDefbox *defbox, gchar *def) {
 
         /* ... handle special characters ... */
         if (*p == '\\') {
-            /*font = (font == gdict_pref.typefaces[TYPEFACE_BODY].font) ? \
-                gdict_pref.typefaces[TYPEFACE_PRONUNCIATION].font : \
-                gdict_pref.typefaces[TYPEFACE_BODY].font;*/
-            color = (color == gdict_pref.typefaces[TYPEFACE_BODY].color) ? \
-                gdict_pref.typefaces[TYPEFACE_PRONUNCIATION].color : \
-                gdict_pref.typefaces[TYPEFACE_BODY].color;
-            if (font == gdict_pref.typefaces[TYPEFACE_BODY].font) {
-                font = gdict_pref.typefaces[TYPEFACE_PRONUNCIATION].font;
+            /*font = (font == gdict->pref->typefaces[TYPEFACE_BODY].font) ? \
+                gdict->pref->typefaces[TYPEFACE_PRONUNCIATION].font : \
+                gdict->pref->typefaces[TYPEFACE_BODY].font;*/
+            color = (color == gdict->pref->typefaces[TYPEFACE_BODY].color) ? \
+                gdict->pref->typefaces[TYPEFACE_PRONUNCIATION].color : \
+                gdict->pref->typefaces[TYPEFACE_BODY].color;
+            if (font == gdict->pref->typefaces[TYPEFACE_BODY].font) {
+                font = gdict->pref->typefaces[TYPEFACE_PRONUNCIATION].font;
                 bold = FALSE;
                 italic = TRUE;
                 underline = FALSE;
             }
             else {
-            	font = gdict_pref.typefaces[TYPEFACE_BODY].font;
+            	font = gdict->pref->typefaces[TYPEFACE_BODY].font;
             	bold = FALSE;
             	italic = FALSE;
             	underline = FALSE;
             }
             ++p;
         } else if (*p == '[') {
-            font = gdict_pref.typefaces[TYPEFACE_ETYMOLOGY].font;
-            color = gdict_pref.typefaces[TYPEFACE_ETYMOLOGY].color;
+            font = gdict->pref->typefaces[TYPEFACE_ETYMOLOGY].font;
+            color = gdict->pref->typefaces[TYPEFACE_ETYMOLOGY].color;
             bold = FALSE;
             italic = TRUE;
             underline = FALSE;
             ++p;
         } else if (*p == '{') {
-            font = gdict_pref.typefaces[TYPEFACE_EXAMPLE].font;
-            color = gdict_pref.typefaces[TYPEFACE_EXAMPLE].color;
+            font = gdict->pref->typefaces[TYPEFACE_EXAMPLE].font;
+            color = gdict->pref->typefaces[TYPEFACE_EXAMPLE].color;
             bold = TRUE;
             italic = TRUE;
             underline = FALSE;
@@ -609,8 +613,8 @@ defbox_add (GDictDefbox *defbox, gchar *def) {
 			p += length;
 			length = 0;
         } else if (*p == ']' ) {
-            font = gdict_pref.typefaces[TYPEFACE_BODY].font;
-            color = gdict_pref.typefaces[TYPEFACE_BODY].color;
+            font = gdict->pref->typefaces[TYPEFACE_BODY].font;
+            color = gdict->pref->typefaces[TYPEFACE_BODY].color;
             bold = FALSE;
             italic = FALSE;
             underline = FALSE;
@@ -619,22 +623,22 @@ defbox_add (GDictDefbox *defbox, gchar *def) {
         
         len = strcspn (p, " \\[]{}");
 
-        if (font == gdict_pref.typefaces[TYPEFACE_BODY].font) {
+        if (font == gdict->pref->typefaces[TYPEFACE_BODY].font) {
             if (is_xref(p, len)) {
-                font = gdict_pref.typefaces[TYPEFACE_XREF].font;
-                color = gdict_pref.typefaces[TYPEFACE_XREF].color;
+                font = gdict->pref->typefaces[TYPEFACE_XREF].font;
+                color = gdict->pref->typefaces[TYPEFACE_XREF].color;
                 bold = FALSE;
                 italic = FALSE;
                 underline = FALSE;
             } else if (is_number(p, len)) {
-                font = gdict_pref.typefaces[TYPEFACE_NUMBER].font;
-                color = gdict_pref.typefaces[TYPEFACE_NUMBER].color;
+                font = gdict->pref->typefaces[TYPEFACE_NUMBER].font;
+                color = gdict->pref->typefaces[TYPEFACE_NUMBER].color;
                 bold = TRUE;
                 italic = FALSE;
                 underline = FALSE;
             } else if (is_part(p, len)) {
-                font = gdict_pref.typefaces[TYPEFACE_PART].font;
-                color = gdict_pref.typefaces[TYPEFACE_PART].color;
+                font = gdict->pref->typefaces[TYPEFACE_PART].font;
+                color = gdict->pref->typefaces[TYPEFACE_PART].color;
                 bold = FALSE;
                 italic = FALSE;
                 underline = FALSE;
@@ -653,31 +657,28 @@ static void
 def_error_cb (dict_command_t *command, DictStatusCode code,
               gchar *message, gpointer data)
 {
-    GtkObject *defbox;
+    GObject *defbox;
     dict_command_t *sec_cmd;
-    gchar *string;
-    
-    defbox = GTK_OBJECT (data);
-    
+
+    defbox = G_OBJECT (data);
+
     if (code != DICT_SOCKET_ERROR) {
         GtkWidget *dialog;
         
-        string = g_strdup_printf (_("Error invoking query: %s"), message);
-        dialog = gtk_message_dialog_new (NULL, GTK_DIALOG_DESTROY_WITH_PARENT,
-                                  	 GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
-                                  	 "%s", string); 
+        dialog = gtk_message_dialog_new_with_markup (NULL, GTK_DIALOG_DESTROY_WITH_PARENT,
+						     GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
+						     "<span weight=\"bold\" size=\"larger\">%s</span>\n\n%s",
+						     _("Error invoking query"), message);
 	gtk_dialog_run (GTK_DIALOG (dialog));
 	gtk_widget_destroy (dialog);
 	
-	g_free (string);
         sec_cmd = dict_disconnect_command_new ();
         dict_command_invoke (sec_cmd, command->context);
         
-        gtk_signal_emit (defbox, gdict_defbox_signals[WORD_LOOKUP_DONE_SIGNAL]);
+        g_signal_emit (defbox, gdict_defbox_signals[WORD_LOOKUP_DONE_SIGNAL], 0);
     }
     else {
-        gtk_signal_emit (defbox, gdict_defbox_signals[SOCKET_ERROR_SIGNAL],
-                         message);
+        g_signal_emit (defbox, gdict_defbox_signals[SOCKET_ERROR_SIGNAL], 0, message);
     }
 }
 
@@ -687,7 +688,8 @@ def_error_cb (dict_command_t *command, DictStatusCode code,
  */
 
 static void 
-def_data_cb (dict_command_t *command, dict_res_t *res, gpointer data) {
+def_data_cb (dict_command_t *command, dict_res_t *res, gpointer data)
+{
     GDictDefbox *defbox;
     
     defbox = GDICT_DEFBOX (data);
@@ -703,53 +705,12 @@ static void
 def_status_cb (dict_command_t *command, DictStatusCode code, 
                int num_found, gpointer data)
 {
-    GtkObject *defbox;
-    
-    defbox = GTK_OBJECT (data);
-    
+    GObject *defbox;
+
+    defbox = G_OBJECT (data);
+
     if (code == DICT_STATUS_OK)
-      gtk_signal_emit (defbox, gdict_defbox_signals[WORD_LOOKUP_DONE_SIGNAL]);
+        g_signal_emit (defbox, gdict_defbox_signals[WORD_LOOKUP_DONE_SIGNAL], 0);
     else if (code == DICT_STATUS_NO_MATCH)
-      gtk_signal_emit (defbox, gdict_defbox_signals[WORD_NOT_FOUND_SIGNAL]);
+        g_signal_emit (defbox, gdict_defbox_signals[WORD_NOT_FOUND_SIGNAL], 0);
 }
-
-#ifdef HAVE_GNOME_PRINT
-
-/* get_line
- *
- * Formats a single line of text for printing and returns it
- */
-
-static GnomeTextLine *
-get_line (const char *fontlist, const char *string, double size) {
-    GnomeTextAttrEl attrs[4];
-    int n_chars, i;
-    GnomeTextLayout *layout;
-    GnomeTextLine *line;
-
-    n_chars = 0;
-    for (i = 0; string[i]; i++)
-      if ((string[i] & 0xc0) != 0x80)
-        n_chars++;
-
-    i = 0;
-    attrs[i].char_pos = 0;
-    attrs[i].attr = GNOME_TEXT_FONT_LIST;
-    attrs[i++].attr_val = gnome_text_intern_font_list (fontlist);
-    attrs[i].char_pos = 0;
-    attrs[i].attr = GNOME_TEXT_SIZE;
-    attrs[i++].attr_val = floor (size * 1000 + 0.5);
-    attrs[i].char_pos = n_chars;
-    attrs[i].attr = GNOME_TEXT_END;
-    attrs[i++].attr_val = 0;
-
-    layout = gnome_text_layout_new (string, attrs);
-
-    line = gnome_text_line_from_layout (layout);
-
-    gnome_text_layout_free (layout);
-
-    return line;
-}
-
-#endif /* HAVE_GNOME_PRINT */
