@@ -1,4 +1,4 @@
-/* $Id: gdict-pref-dialog.c,v 1.45 2004/11/24 22:15:27 vnoel Exp $ */
+/* $Id: gdict-pref-dialog.c,v 1.57 2004/12/17 17:15:17 vnoel Exp $ */
 
 /*
  *  Mike Hughes <mfh@psilord.com>
@@ -22,6 +22,8 @@
 #include "gdict-pref-dialog.h"
 #include "gdict-app.h"
 #include "gdict-utils.h"
+#include "gdict-speller.h"
+#include "gdict-web-search.h"
 
 static void gdict_pref_dialog_class_init (GDictPrefDialogClass *class);
 static void gdict_pref_dialog_init (GDictPrefDialog *pref_dialog);
@@ -51,14 +53,20 @@ static gboolean	set_server_cb 	    (GtkWidget *widget, GdkEventFocus *event, gpo
 static gboolean set_port_cb	    (GtkWidget *widget, GdkEventFocus *event, gpointer data);
 static void set_default_server      (GtkButton *button, gpointer data);
 static void set_default_port        (GtkButton *button, gpointer data);
-static void smart_lookup_cb	    (GtkToggleButton *toggle, gpointer data);
-
 
 enum {
     SOCKET_ERROR_SIGNAL,
     LAST_SIGNAL
 };
 
+enum {
+	ENABLED = 0,
+	WEBSITE,
+	URL,
+	POINTER_COLUMN,
+	NUM_COLUMNS
+};
+
 static gint gdict_pref_dialog_signals[LAST_SIGNAL] = { 0 };
 
 static GtkDialogClass *parent_class = NULL;
@@ -68,7 +76,6 @@ GtkWidget *error_label2 = NULL;
 GtkWidget *alignment1 = NULL;
 GtkWidget *alignment2 = NULL;
 
-
 GType
 gdict_pref_dialog_get_type (void)
 {
@@ -120,39 +127,124 @@ gdict_pref_dialog_class_init (GDictPrefDialogClass *class)
     object_class->finalize = gdict_pref_dialog_finalize;
 }
 
-static void 
-gdict_pref_dialog_init (GDictPrefDialog *pref_dialog)
+static web_site_edited (GtkCellRendererText *cell,
+			gchar *path_str, gchar *new_text,
+			gpointer data)
 {
-    GtkWidget *server_label, *port_label;
-    GtkWidget *vbox, *hbox;
-    GtkWidget *button;    
-    gchar *port_str;
-    
-    pref_dialog->context = NULL;
+	GDictPrefDialog *pref_dialog = data;
+	GDictWindow *gdict = pref_dialog->gdict;
+	GtkTreeModel *model;
+	GtkTreePath *path = gtk_tree_path_new_from_string (path_str);
+	GtkTreeIter iter;
+	GDictWebSearchSite *site;
+	gint column = (gint) g_object_get_data (G_OBJECT (cell), "column");
+	
+	model = gtk_tree_view_get_model (GTK_TREE_VIEW (pref_dialog->view));
+	gtk_tree_model_get_iter (model, &iter, path);
+	gtk_tree_model_get (model, &iter, POINTER_COLUMN, &site, -1);
 
-    pref_dialog->database = gdict_pref.database;
-    pref_dialog->dfl_strat = gdict_pref.dfl_strat;
-    
-    gtk_window_set_title (GTK_WINDOW (pref_dialog), _("Dictionary Preferences"));
-    gtk_window_set_resizable (GTK_WINDOW (pref_dialog), FALSE);
-    gtk_container_set_border_width (GTK_CONTAINER (pref_dialog), 5);
-    vbox = GTK_DIALOG (pref_dialog)->vbox;
+	if (column == WEBSITE) {
+		g_free (site->name);
+		site->name = g_strdup_printf ("%s", new_text);
+	} else {
+		g_free (site->url);
+		site->url = g_strdup_printf ("%s", new_text);
+	}
+	gtk_list_store_set (GTK_LIST_STORE (model), &iter, column, new_text, -1);
+	
+}
+
+static web_site_toggled (GtkCellRendererToggle *cell,
+			 gchar                 *path_str,
+			 gpointer               data)
+{
+	GDictPrefDialog *pref_dialog = data;
+	GDictWindow *gdict = pref_dialog->gdict;
+	GtkTreePath *path = gtk_tree_path_new_from_string (path_str);
+	GtkTreeModel *model;
+	GtkTreeIter iter;
+	gboolean toggle_item;
+	GDictWebSearchSite *site;
+	
+	model = gtk_tree_view_get_model (GTK_TREE_VIEW (pref_dialog->view));
+	gtk_tree_model_get_iter (model, &iter, path);
+	gtk_tree_model_get (model, &iter, ENABLED, &toggle_item, POINTER_COLUMN, &site, -1);
+
+	toggle_item ^= 1;
+	site->enabled = toggle_item;
+
+	gtk_cell_renderer_toggle_set_active (cell, toggle_item);
+	gtk_list_store_set (GTK_LIST_STORE (model), &iter, ENABLED, toggle_item, -1);
+}
+
+static web_site_add (GtkButton *widget, gpointer data)
+{
+	GDictWebSearchSite *site;
+	GDictPrefDialog *pref_dialog = data;
+	GDictWindow *gdict = pref_dialog->gdict;
+	GtkTreeModel *model;
+	GtkTreeIter iter;
+	GtkTreeSelection *selection;
+	GtkTreePath *path;
+	GtkTreeViewColumn *column;
+
+	site = gdict_web_search_new ("", "http://", TRUE);
+	gdict->web_search_sites = g_slist_prepend (gdict->web_search_sites, site);
+	model = gtk_tree_view_get_model (GTK_TREE_VIEW (pref_dialog->view));
+	gtk_list_store_prepend (GTK_LIST_STORE(model), &iter);
+	gtk_list_store_set (GTK_LIST_STORE(model), &iter, 
+			    ENABLED, site->enabled,
+			    WEBSITE, site->name,
+			    URL, site->url, 
+			    POINTER_COLUMN, site,
+			    -1);
+
+	path = gtk_tree_model_get_path (model, &iter);
+	column = gtk_tree_view_get_column (GTK_TREE_VIEW (pref_dialog->view), WEBSITE);
+	gtk_tree_view_set_cursor (GTK_TREE_VIEW (pref_dialog->view), path, column, TRUE);
+	gtk_tree_path_free (path);
+}
+
+static web_site_remove (GtkButton *widget, gpointer data)
+{
+	GDictPrefDialog *pref_dialog = data;
+	GDictWindow *gdict = pref_dialog->gdict;
+	GtkTreeSelection *selection;
+	GtkTreeModel *model;
+	GtkTreeIter iter;
+	GDictWebSearchSite *site;
+
+	model = gtk_tree_view_get_model (GTK_TREE_VIEW (pref_dialog->view));
+	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (pref_dialog->view));
+	gtk_tree_selection_get_selected (selection, &model, &iter);
+	gtk_tree_model_get (model, &iter, POINTER_COLUMN, &site, -1);
+	gdict_web_search_remove (gdict, site);
+	gtk_list_store_remove (GTK_LIST_STORE (model), &iter);
+}
+
+static web_site_selected (GtkTreeSelection *selection, gpointer data)
+{
+	GDictPrefDialog *pref_dialog = data;
+	if (gtk_tree_selection_count_selected_rows (selection) > 0) 
+		gtk_widget_set_sensitive (pref_dialog->remove_button, TRUE);
+	else
+		gtk_widget_set_sensitive (pref_dialog->remove_button, FALSE);
+}
+
+static GtkWidget *
+create_dictionary_panel (GDictPrefDialog *pref_dialog)
+{
+    GtkWidget *vbox, *hbox;
+
+    vbox = gtk_vbox_new (FALSE, 0);
     gtk_box_set_spacing (GTK_BOX (vbox), 2);
+    gtk_container_set_border_width (GTK_CONTAINER (vbox), 5);
     
-    pref_dialog->table = GTK_TABLE (gtk_table_new (5, 2, FALSE));
-    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (pref_dialog)->vbox), GTK_WIDGET (pref_dialog->table), TRUE, TRUE, 0);
+    pref_dialog->table = GTK_TABLE (gtk_table_new (4, 2, FALSE));
+    gtk_box_pack_start (GTK_BOX (vbox), GTK_WIDGET (pref_dialog->table), FALSE, FALSE, 0);
     gtk_container_set_border_width (GTK_CONTAINER (pref_dialog->table), 5);
     gtk_table_set_row_spacings (pref_dialog->table, 6);
     gtk_table_set_col_spacings (pref_dialog->table, 12);
-
-    pref_dialog->smart_lookup_btn = 
-        GTK_CHECK_BUTTON (gtk_check_button_new_with_mnemonic (_("Smart _lookup")));
-    g_signal_connect (G_OBJECT (pref_dialog->smart_lookup_btn), "toggled",
-    		      G_CALLBACK (smart_lookup_cb), pref_dialog);
-    gtk_table_attach_defaults (pref_dialog->table, GTK_WIDGET (pref_dialog->smart_lookup_btn),
-    			       1, 2, 2, 3);
-    if ( ! gconf_client_key_is_writable (gdict_get_gconf_client (), "/apps/gnome-dictionary/smart", NULL))
-	    gtk_widget_set_sensitive (GTK_WIDGET (pref_dialog->smart_lookup_btn), FALSE);
     
     hbox = gtk_hbox_new (FALSE, 6);
     pref_dialog->server_entry = GTK_ENTRY (gtk_entry_new ());	
@@ -160,9 +252,9 @@ gdict_pref_dialog_init (GDictPrefDialog *pref_dialog)
     g_signal_connect (G_OBJECT (pref_dialog->server_entry), "focus_out_event",
     		      G_CALLBACK (set_server_cb), pref_dialog);
 
-    button = gtk_button_new_with_mnemonic (_("De_fault Server"));
-    gtk_box_pack_start (GTK_BOX (hbox), GTK_WIDGET(button), FALSE, FALSE, 0);
-    g_signal_connect (G_OBJECT (button), "clicked",
+    pref_dialog->server_button = gtk_button_new_with_mnemonic (_("De_fault Server"));
+    gtk_box_pack_start (GTK_BOX (hbox), GTK_WIDGET(pref_dialog->server_button), FALSE, FALSE, 0);
+    g_signal_connect (G_OBJECT (pref_dialog->server_button), "clicked",
     		      G_CALLBACK (set_default_server), pref_dialog);
 
     gtk_table_attach (pref_dialog->table, 
@@ -171,40 +263,33 @@ gdict_pref_dialog_init (GDictPrefDialog *pref_dialog)
                       (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                       (GtkAttachOptions) (0), 0, 0);
 		    
-    server_label = gtk_label_new_with_mnemonic (_("_Server:"));
-    gtk_label_set_mnemonic_widget (GTK_LABEL (server_label), 
+    pref_dialog->server_label = gtk_label_new_with_mnemonic (_("_Server:"));
+    gtk_label_set_mnemonic_widget (GTK_LABEL (pref_dialog->server_label), 
     				   GTK_WIDGET (pref_dialog->server_entry));
-    gtk_label_set_justify (GTK_LABEL (server_label), GTK_JUSTIFY_LEFT);
-    gtk_misc_set_alignment (GTK_MISC (server_label), 0, 0.5);
-    gtk_table_attach_defaults (pref_dialog->table, server_label, 0, 1, 0, 1);
+    gtk_label_set_justify (GTK_LABEL (pref_dialog->server_label), GTK_JUSTIFY_LEFT);
+    gtk_misc_set_alignment (GTK_MISC (pref_dialog->server_label), 0, 0.5);
+    gtk_table_attach_defaults (pref_dialog->table, pref_dialog->server_label, 0, 1, 0, 1);
     
-
     if (gail_loaded)
-        gdict_add_atk_namedesc (GTK_WIDGET (button), NULL, _("Reset server to default"));
+        gdict_add_atk_namedesc (GTK_WIDGET (pref_dialog->server_button), NULL, _("Reset server to default"));
 
-    if ( ! gconf_client_key_is_writable (gdict_get_gconf_client (), "/apps/gnome-dictionary/server", NULL)) {
-	    gtk_widget_set_sensitive (GTK_WIDGET (pref_dialog->server_entry), FALSE);
-	    gtk_widget_set_sensitive (server_label, FALSE);
-	    gtk_widget_set_sensitive (button, FALSE);
-    }
-
-    port_label = gtk_label_new_with_mnemonic (_("_Port:"));
-    gtk_label_set_justify (GTK_LABEL (port_label), GTK_JUSTIFY_LEFT);
-    gtk_misc_set_alignment (GTK_MISC (port_label), 0, 0.5);
-    gtk_table_attach_defaults (pref_dialog->table, port_label, 0, 1, 1, 2);
+    pref_dialog->port_label = gtk_label_new_with_mnemonic (_("_Port:"));
+    gtk_label_set_justify (GTK_LABEL (pref_dialog->port_label), GTK_JUSTIFY_LEFT);
+    gtk_misc_set_alignment (GTK_MISC (pref_dialog->port_label), 0, 0.5);
+    gtk_table_attach_defaults (pref_dialog->table, pref_dialog->port_label, 0, 1, 1, 2);
 
     pref_dialog->port_entry = GTK_ENTRY (gtk_entry_new ());
-    gtk_label_set_mnemonic_widget (GTK_LABEL (port_label), 
+    gtk_label_set_mnemonic_widget (GTK_LABEL (pref_dialog->port_label), 
     				   GTK_WIDGET (pref_dialog->port_entry));
     g_signal_connect (G_OBJECT (pref_dialog->port_entry), "focus_out_event",
     		      G_CALLBACK (set_port_cb), pref_dialog);
-    button = gtk_button_new_with_mnemonic (_("Def_ault Port"));
-    g_signal_connect (G_OBJECT (button), "clicked",
+    pref_dialog->port_button = gtk_button_new_with_mnemonic (_("Def_ault Port"));
+    g_signal_connect (G_OBJECT (pref_dialog->port_button), "clicked",
     		      G_CALLBACK (set_default_port), pref_dialog);
 
     hbox = gtk_hbox_new (FALSE, 6);
     gtk_box_pack_start (GTK_BOX (hbox), GTK_WIDGET(pref_dialog->port_entry), TRUE, TRUE, 0);
-    gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
+    gtk_box_pack_start (GTK_BOX (hbox), pref_dialog->port_button, FALSE, FALSE, 0);
     gtk_table_attach (pref_dialog->table, 
                       hbox,
                       1, 2, 1, 2,
@@ -213,37 +298,185 @@ gdict_pref_dialog_init (GDictPrefDialog *pref_dialog)
 
 
     if (gail_loaded) {
-        gdict_add_atk_namedesc (GTK_WIDGET(button), NULL, _("Reset port to default"));
-        gdict_add_atk_namedesc (server_label, _("Server"), _("Server"));
+        gdict_add_atk_namedesc (GTK_WIDGET(pref_dialog->port_button), NULL, _("Reset port to default"));
+        gdict_add_atk_namedesc (pref_dialog->server_label, _("Server"), _("Server"));
         gdict_add_atk_namedesc (GTK_WIDGET (pref_dialog->server_entry),
 				_("Server Entry"), _("Enter the Server Name"));
-        gdict_add_atk_namedesc (port_label, _("Port"), _("Port"));
+        gdict_add_atk_namedesc (pref_dialog->port_label, _("Port"), _("Port"));
         gdict_add_atk_namedesc (GTK_WIDGET (pref_dialog->port_entry),
 				_("Port Entry"), _("Enter the Port Number"));
     }
 
-    if ( ! gconf_client_key_is_writable (gdict_get_gconf_client (), "/apps/gnome-dictionary/port", NULL)) {
-	    gtk_widget_set_sensitive (GTK_WIDGET (pref_dialog->port_entry), FALSE);
-	    gtk_widget_set_sensitive (port_label, FALSE);
-	    gtk_widget_set_sensitive (button, FALSE);
-    }
-
     pref_dialog->db_label = gtk_label_new_with_mnemonic (_("_Database:"));
     gtk_label_set_justify (GTK_LABEL (pref_dialog->db_label), GTK_JUSTIFY_LEFT);
     gtk_misc_set_alignment (GTK_MISC (pref_dialog->db_label), 0, 0.5);
-    gtk_table_attach_defaults (pref_dialog->table, pref_dialog->db_label, 0, 1, 3, 4);
+    gtk_table_attach_defaults (pref_dialog->table, pref_dialog->db_label, 0, 1, 2, 3);
     pref_dialog->db_list = GTK_MENU (gtk_menu_new ());
     gtk_widget_show (GTK_WIDGET (pref_dialog->db_list));
     
-    pref_dialog->strat_label = gtk_label_new_with_mnemonic (_("D_efault strategy:"));
+    pref_dialog->strat_label = gtk_label_new_with_mnemonic (_("Strat_egy:"));
     gtk_label_set_justify (GTK_LABEL (pref_dialog->strat_label), GTK_JUSTIFY_LEFT);
     gtk_misc_set_alignment (GTK_MISC (pref_dialog->strat_label), 0, 0.5);
-    gtk_table_attach_defaults (pref_dialog->table, pref_dialog->strat_label, 0, 1, 4, 5);
+    gtk_table_attach_defaults (pref_dialog->table, pref_dialog->strat_label, 0, 1, 3, 4);
     pref_dialog->strat_list = GTK_MENU (gtk_menu_new ());
     gtk_widget_show (GTK_WIDGET (pref_dialog->strat_list));
     
     gtk_widget_show_all (GTK_WIDGET (pref_dialog->table));
 
+    return vbox;
+}
+
+static GtkWidget *
+create_websites_panel (GDictPrefDialog *pref_dialog)
+{
+	int i;
+
+	GtkWidget *vbox, *hbox;
+	GtkWidget *button;
+	GtkWidget *scrolled;
+	GtkListStore *store;
+	GtkTreeSelection *selection;
+	GtkWidget *view;
+	GtkTreeViewColumn *column;
+	GtkCellRenderer *renderer;
+		
+	const gchar *column_titles[] = { N_(""), N_("Website"), N_("Search Address"), NULL };
+
+	vbox = gtk_vbox_new (FALSE, 5);
+	gtk_container_set_border_width (GTK_CONTAINER (vbox), 5);
+
+	scrolled = gtk_scrolled_window_new (NULL, NULL);
+	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled), 
+					GTK_POLICY_AUTOMATIC,
+					GTK_POLICY_AUTOMATIC);
+	gtk_box_pack_start (GTK_BOX (vbox), scrolled, TRUE, TRUE, 0);
+
+	store = gtk_list_store_new (NUM_COLUMNS, G_TYPE_BOOLEAN, G_TYPE_STRING, G_TYPE_STRING,
+				    G_TYPE_POINTER, G_TYPE_BOOLEAN, G_TYPE_BOOLEAN);	
+	view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (store));
+	gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (view), TRUE);
+	g_object_unref (G_OBJECT (store));
+	
+	renderer = gtk_cell_renderer_toggle_new ();
+	g_object_set (G_OBJECT (renderer), "activatable", TRUE, NULL);
+	g_signal_connect (renderer, "toggled", G_CALLBACK (web_site_toggled), pref_dialog);
+
+	column = gtk_tree_view_column_new_with_attributes (_(column_titles[0]), renderer, 
+							   "active", ENABLED, NULL);
+	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
+	gtk_tree_view_column_set_resizable (column, FALSE);
+	gtk_tree_view_column_set_clickable (column, TRUE);
+	gtk_tree_view_append_column (GTK_TREE_VIEW (view), column);
+
+	for (i=1; column_titles[i]; i++) {
+		renderer = gtk_cell_renderer_text_new ();
+		g_object_set (G_OBJECT (renderer), "editable", TRUE, NULL);
+		g_object_set_data (G_OBJECT (renderer), "column", (gint *) i);
+		g_signal_connect (renderer, "edited", G_CALLBACK (web_site_edited), pref_dialog);
+		column = gtk_tree_view_column_new_with_attributes (_(column_titles[i]),
+								   renderer,
+								   "text", i,
+								   NULL);
+		gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
+		gtk_tree_view_column_set_resizable (column, FALSE);
+		gtk_tree_view_append_column (GTK_TREE_VIEW (view), column);
+		}
+	gtk_widget_set_size_request (GTK_WIDGET (view), 400,200);
+	gtk_container_add (GTK_CONTAINER (scrolled), GTK_WIDGET (view));
+
+	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (view));
+	g_signal_connect (selection, "changed", G_CALLBACK(web_site_selected), pref_dialog);
+	gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);
+
+	hbox = gtk_hbutton_box_new ();
+	gtk_button_box_set_layout (GTK_BUTTON_BOX (hbox), GTK_BUTTONBOX_START);
+	gtk_button_box_set_spacing (GTK_BUTTON_BOX (hbox), 5);
+	button = gtk_button_new_from_stock (GTK_STOCK_ADD);
+	g_signal_connect (button, "clicked", G_CALLBACK (web_site_add), pref_dialog);
+	gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
+	button = gtk_button_new_from_stock (GTK_STOCK_REMOVE);
+	gtk_widget_set_sensitive (button, FALSE);
+	g_signal_connect (button, "clicked", G_CALLBACK (web_site_remove), pref_dialog);
+	gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
+	
+	gtk_box_pack_end (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
+
+	pref_dialog->view = view;
+	pref_dialog->remove_button = button;
+
+	return vbox;
+}
+
+static void
+gdict_pref_dialog_set (GDictPrefDialog *pref_dialog)
+{
+	gchar *port_str;
+	GDictPref *pref = pref_dialog->pref;
+	GDictWindow *gdict = pref_dialog->gdict;
+	GtkTreeIter iter;
+	GtkTreeModel *model;
+	GSList *site_list;
+	GDictWebSearchSite *site;
+
+	pref_dialog->database = pref->database;
+	pref_dialog->dfl_strat = pref->dfl_strat;
+	if (pref->server != NULL)
+		gtk_entry_set_text (GTK_ENTRY (pref_dialog->server_entry),
+				    pref->server);
+	port_str = g_strdup_printf("%d", (int) pref->port);
+	gtk_entry_set_text (GTK_ENTRY (pref_dialog->port_entry), port_str);
+	g_free(port_str);
+        if (! gdict_is_gconf_key_writable (pref_dialog->gdict, "port")) {
+                gtk_widget_set_sensitive (GTK_WIDGET (pref_dialog->port_entry), FALSE);
+                gtk_widget_set_sensitive (pref_dialog->port_label, FALSE);
+                gtk_widget_set_sensitive (pref_dialog->port_button, FALSE);
+        }
+        if ( ! gdict_is_gconf_key_writable (pref_dialog->gdict, "server")) {
+                gtk_widget_set_sensitive (GTK_WIDGET (pref_dialog->server_entry), FALSE);
+                gtk_widget_set_sensitive (pref_dialog->server_label, FALSE);
+                gtk_widget_set_sensitive (pref_dialog->server_button, FALSE);
+        }
+
+	/* add the websites to the list */
+	
+	model = gtk_tree_view_get_model (GTK_TREE_VIEW (pref_dialog->view));
+	for (site_list = gdict->web_search_sites; site_list; site_list = g_slist_next (site_list)) {
+		site = site_list->data;
+		gtk_list_store_prepend (GTK_LIST_STORE(model), &iter);
+		gtk_list_store_set (GTK_LIST_STORE(model), &iter, 
+				    ENABLED, site->enabled,
+				    WEBSITE, site->name,
+				    URL, site->url, 
+				    POINTER_COLUMN, site,
+				    -1);
+	}
+
+}
+
+static void 
+gdict_pref_dialog_init (GDictPrefDialog *pref_dialog)
+{
+    GtkWidget *vbox;
+    GtkWidget *notebook;
+    GtkWidget *label;
+
+    gtk_window_set_title (GTK_WINDOW (pref_dialog), _("Dictionary Preferences"));
+    gtk_window_set_resizable (GTK_WINDOW (pref_dialog), TRUE);
+    gtk_container_set_border_width (GTK_CONTAINER (pref_dialog), 5);
+
+    notebook = gtk_notebook_new ();
+ 
+    vbox = create_dictionary_panel (pref_dialog);
+    label = gtk_label_new (_("Dictionary"));
+    gtk_notebook_append_page (GTK_NOTEBOOK (notebook), vbox, label);
+
+    vbox = create_websites_panel (pref_dialog);
+    label = gtk_label_new (_("Web sites"));
+    gtk_notebook_append_page (GTK_NOTEBOOK (notebook), vbox, label);
+
+    gtk_container_add (GTK_CONTAINER(GTK_DIALOG (pref_dialog)->vbox), notebook);
+    gtk_container_set_border_width (GTK_CONTAINER (notebook), 5);
+
     gtk_dialog_add_buttons (GTK_DIALOG (pref_dialog), GTK_STOCK_CLOSE,
     			    GTK_RESPONSE_CLOSE, GTK_STOCK_HELP, GTK_RESPONSE_HELP, NULL);  
 
@@ -256,17 +489,6 @@ gdict_pref_dialog_init (GDictPrefDialog *pref_dialog)
     /* To prevent dialog desctruction when pressing ESC */
     g_signal_connect (G_OBJECT (pref_dialog), "delete_event", 
 		      G_CALLBACK (gtk_true), NULL);
-
-    if (gdict_pref.server != NULL)
-	    gtk_entry_set_text (GTK_ENTRY (pref_dialog->server_entry),
-				gdict_pref.server);
-    port_str = g_strdup_printf("%d", (int) gdict_pref.port);
-    gtk_entry_set_text (GTK_ENTRY (pref_dialog->port_entry), port_str);
-    g_free(port_str);
-    gtk_toggle_button_set_active
-	(GTK_TOGGLE_BUTTON (pref_dialog->smart_lookup_btn), gdict_pref.smart);
-
-    gtk_widget_show_all (GTK_WIDGET (pref_dialog));
 }
 
 static void
@@ -284,38 +506,47 @@ gdict_pref_dialog_finalize (GObject *object)
 }
 
 GtkWidget *
-gdict_pref_dialog_new (dict_context_t *context) 
+gdict_pref_dialog_new (GDictWindow *gdict, GDictPref *pref)
 {
     GDictPrefDialog *pref_dialog;
+    dict_context_t *context = gdict->context;
     
-    /* g_return_val_if_fail (context != NULL, NULL); */
-    /* the above is commented out because it prints ugly debug messages
-     * to the console */
     if (context == NULL)
 	    return NULL;
 
     pref_dialog = GDICT_PREF_DIALOG (g_object_new (GDICT_TYPE_PREF_DIALOG, NULL));
-    pref_dialog->context = context;    
+    pref_dialog->gdict = gdict;
+    pref_dialog->pref = pref;
+    pref_dialog->context = context;
+    gdict_pref_dialog_set (pref_dialog);
+
     pref_dialog_reset_db (pref_dialog);
     pref_dialog_reset_strat (pref_dialog);
+    
+    gtk_widget_show_all (GTK_WIDGET (pref_dialog));
 
     return GTK_WIDGET (pref_dialog);
 }
 
 /* pref_set_db_cb
  *
- * Sets the current search strategy to the one indicated
+ * Sets the current search database to the one indicated
  */
 
 static void
 pref_set_db_cb (GtkWidget *widget, gpointer data) 
 {
     GDictPrefDialog *pref_dialog;
+    GDictWindow *gdict;
 
     pref_dialog = GDICT_PREF_DIALOG (data);
+    gdict = pref_dialog->gdict;
     pref_dialog->database = g_object_get_data (G_OBJECT (widget), "db_name");
+    pref_dialog->pref->database = pref_dialog->database;
 
-    gconf_client_set_string (gdict_get_gconf_client (), "/apps/gnome-dictionary/database", pref_dialog->database, NULL);
+    gdict_set_gconf_string (pref_dialog->gdict, "database", pref_dialog->database);
+    if (gdict->speller_visible)
+	    gdict_speller_change_db (gdict->speller, pref_dialog->database);
 }
 
 /* pref_set_strat_cb
@@ -327,11 +558,16 @@ static void
 pref_set_strat_cb (GtkWidget *widget, gpointer data) 
 {
     GDictPrefDialog *pref_dialog;
+    GDictWindow *gdict;
     
     pref_dialog = GDICT_PREF_DIALOG (data);
+    gdict = pref_dialog->gdict;
     pref_dialog->dfl_strat = g_object_get_data (G_OBJECT (widget), "strat_name");
+    pref_dialog->pref->dfl_strat = pref_dialog->dfl_strat;
 
-    gconf_client_set_string (gdict_get_gconf_client (), "/apps/gnome-dictionary/strategy", pref_dialog->dfl_strat, NULL);
+    gdict_set_gconf_string (gdict, "strategy", pref_dialog->dfl_strat);
+    if (gdict->speller_visible)
+	    gdict_speller_change_strategy (gdict->speller, pref_dialog->dfl_strat);
 }
 
 /* pref_dialog_add_db
@@ -429,7 +665,7 @@ pref_dialog_reset_db (GDictPrefDialog *pref_dialog)
 	error_label1 = GTK_WIDGET (gtk_label_new (_("Cannot connect to server")));
 	alignment1 = GTK_WIDGET (gtk_alignment_new (0, 0.5, 0, 0));
 	gtk_container_add (GTK_CONTAINER (alignment1), error_label1);
-	gtk_table_attach_defaults (pref_dialog->table, alignment1, 1, 2, 3, 4);
+	gtk_table_attach_defaults (pref_dialog->table, alignment1, 1, 2, 2, 3);
 	gtk_widget_show_all (GTK_WIDGET (pref_dialog->table));
     }
 }
@@ -478,7 +714,7 @@ pref_dialog_reset_strat (GDictPrefDialog *pref_dialog)
 	error_label2 = GTK_WIDGET (gtk_label_new (_("Cannot connect to server")));
 	alignment2 = GTK_WIDGET (gtk_alignment_new (0, 0.5, 0, 0));
 	gtk_container_add (GTK_CONTAINER (alignment2), error_label2);
-	gtk_table_attach_defaults (pref_dialog->table, alignment2, 1, 2, 4, 5);
+	gtk_table_attach_defaults (pref_dialog->table, alignment2, 1, 2, 3, 4);
 	gtk_widget_show_all (GTK_WIDGET (pref_dialog->table));
     }
 }
@@ -558,11 +794,11 @@ pref_status_cb (dict_command_t *command, DictStatusCode code,
     
     if (code == DICT_STATUS_OK) {
         if (command->cmd == C_SHOW_DB) {
-            row = 3;
+            row = 2;
             use_menu = pref_dialog->db_list;
         }
         else if (command->cmd == C_SHOW_STRAT) {
-            row = 4;
+            row = 3;
             use_menu = pref_dialog->strat_list;
         }
         
@@ -584,8 +820,7 @@ pref_status_cb (dict_command_t *command, DictStatusCode code,
 					pref_dialog->db_label, ATK_RELATION_LABELLED_BY);
             }
 
-            writable = gconf_client_key_is_writable (gdict_get_gconf_client (),
-						     "/apps/gnome-dictionary/database", NULL);
+            writable = gdict_is_gconf_key_writable (pref_dialog->gdict, "database");
 
 	    gtk_widget_set_sensitive (pref_dialog->db_label, writable);
 	    gtk_widget_set_sensitive (GTK_WIDGET (option_menu), writable);
@@ -602,7 +837,7 @@ pref_status_cb (dict_command_t *command, DictStatusCode code,
 					pref_dialog->strat_label, ATK_RELATION_LABELLED_BY);
             }
 
-            writable = gconf_client_key_is_writable (gdict_get_gconf_client (), "/apps/gnome-dictionary/strategy", NULL);
+            writable = gdict_is_gconf_key_writable (pref_dialog->gdict, "strategy");
 
 	    gtk_widget_set_sensitive (pref_dialog->strat_label, writable);
 	    gtk_widget_set_sensitive (GTK_WIDGET (option_menu), writable);
@@ -646,30 +881,34 @@ static gboolean
 set_server_cb (GtkWidget *widget, GdkEventFocus *event, gpointer data)
 {
     GDictPrefDialog *pref_dialog;
+    GDictPref *pref;
     gchar *server;
 
     g_return_val_if_fail (GDICT_IS_PREF_DIALOG (data), FALSE);
 
     pref_dialog = data;
+    pref = pref_dialog->pref;
     server = gtk_editable_get_chars (GTK_EDITABLE (pref_dialog->server_entry), 0, -1);
     if (!server)
         return FALSE;
 
-    if ((gdict_pref.server == NULL) || (strcmp (gdict_pref.server, server))) {
-	if (gdict_pref.server)
-		g_free (gdict_pref.server);
-        gdict_pref.server = g_strdup(server);
-        gdict_init_context ();
-        pref_dialog->context = context;
-        pref_dialog_reset_db (pref_dialog);
-        pref_dialog_reset_strat (pref_dialog);
-           
-        gconf_client_set_string (gdict_get_gconf_client (), "/apps/gnome-dictionary/server", gdict_pref.server, NULL);
+    if ((pref->server == NULL) || (strcmp (pref->server, server))) {
+            g_free (pref->server);
+            pref->server = g_strdup(server);
+            gdict_init_context (pref_dialog->gdict);
+            pref_dialog->context = pref_dialog->gdict->context;
+            pref_dialog_reset_db (pref_dialog);
+            pref_dialog_reset_strat (pref_dialog);
+            
+            gdict_set_gconf_string (pref_dialog->gdict, "server", pref->server);
 
+	    if (pref_dialog->gdict->speller_visible)
+		    gdict_speller_reset (pref_dialog->gdict->speller, pref_dialog->gdict->context);
     }
 
     g_free (server);
 
+
     return FALSE;
 }
 
@@ -677,23 +916,25 @@ static gboolean
 set_port_cb (GtkWidget *widget, GdkEventFocus *event, gpointer data)
 {
     GDictPrefDialog *pref_dialog;
+    GDictPref *pref;
     gchar *port;
 
     g_return_val_if_fail (GDICT_IS_PREF_DIALOG (data), FALSE);
 
     pref_dialog = data;
+    pref = pref_dialog->pref;
     port = gtk_editable_get_chars (GTK_EDITABLE (pref_dialog->port_entry), 0, -1);
-    if (gdict_pref.port != atoi (port)) {
+    if (pref->port != atoi (port)) {
     
-        gdict_pref.port = atoi (port);
-        gdict_init_context ();
+        pref->port = atoi (port);
+        gdict_init_context (pref_dialog->gdict);
         
-        pref_dialog->context = context;
+        pref_dialog->context = pref_dialog->gdict->context;
         
         pref_dialog_reset_db (pref_dialog);
         pref_dialog_reset_strat (pref_dialog);
             
-        gconf_client_set_int (gdict_get_gconf_client (), "/apps/gnome-dictionary/port", gdict_pref.port, NULL);
+        gdict_set_gconf_int (pref_dialog->gdict, "port", pref->port);
     }
     
     g_free (port);
@@ -726,15 +967,4 @@ set_default_port (GtkButton *button, gpointer data)
     gtk_entry_set_text (GTK_ENTRY (pref_dialog->port_entry), port);
     g_free (port);
     set_port_cb (GTK_WIDGET (pref_dialog->port_entry), NULL, pref_dialog);
-}    
-
-static void 
-smart_lookup_cb	(GtkToggleButton *toggle, gpointer data)
-{
-    gboolean toggled;
-    
-    toggled = gtk_toggle_button_get_active (toggle);
-    if (toggled != gdict_pref.smart)
-        gconf_client_set_bool (gdict_get_gconf_client (), "/apps/gnome-dictionary/smart", toggled, NULL);
 }
-
