@@ -1,7 +1,7 @@
 #ifndef __GDICT_SPELLER_H_
 #define __GDICT_SPELLER_H_
 
-/* $Id: gdict-speller.h,v 1.6 2004/11/04 00:36:37 pborelli Exp $ */
+/* $Id: gdict-speller.h,v 1.11 2004/12/13 20:10:20 vnoel Exp $ */
 
 /*
  *  Mike Hughes <mfh@psilord.com>
@@ -15,13 +15,9 @@
  *
  */
 
-#ifdef __cplusplus
-extern "C" {
-#endif /* __cplusplus */
-
 #include <gtk/gtk.h>
-
 #include "dict.h"
+#include "gdict-app.h"
 
 #define GDICT_TYPE_SPELLER            (gdict_speller_get_type ())
 #define GDICT_SPELLER(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), GDICT_TYPE_SPELLER, GDictSpeller))
@@ -35,27 +31,23 @@ typedef struct _GDictSpeller        GDictSpeller;
 typedef struct _GDictSpellerClass   GDictSpellerClass;
 
 struct _GDictSpeller {
-    GtkDialog      dialog;
+    GtkVBox      vbox;
     
     dict_context_t *context;
-    dict_command_t *get_strat_cmd;
     dict_command_t *spell_cmd;
     gchar          *database;
     gchar          *strat;
     
     GtkWidget      *hbox;
-    GtkEntry       *word_entry;
-    GtkWidget      *strat_label;
-    GtkOptionMenu  *strat_sel;
-    GtkMenu        *strat_list;
-    guint           strat_idx;
     GtkWidget	   *word_list;
     
     gchar          *current_word;
+
+    GDictWindow    *gdict;
 };
 
 struct _GDictSpellerClass {
-    GtkDialogClass  parent_class;
+    GtkVBoxClass  parent_class;
     
     void (*word_lookup_start) (GDictSpeller *);
     void (*word_lookup_done)  (GDictSpeller *);
@@ -64,21 +56,12 @@ struct _GDictSpellerClass {
 };
 
 GType      gdict_speller_get_type    (void);
-
-GtkWidget *gdict_speller_new         ();
-void       gdict_speller_destroy     (GDictSpeller *speller);
-
-gint       gdict_speller_lookup      (GDictSpeller *speller, gchar *text);
+GtkWidget *gdict_speller_new         (GDictWindow *gdict);
+gint       gdict_speller_lookup      (GDictWindow *gdict, GDictSpeller *speller, gchar *text);
 void       gdict_speller_clear       (GDictSpeller *speller);
-void       gdict_speller_reset       (GDictSpeller *speller,
-                                      dict_context_t *context);
-
-gchar     *gdict_speller_get_word    (GDictSpeller *speller);
-
-void       gdict_speller_reset_strat (GDictSpeller *speller);
-
-#ifdef __cplusplus
-}
-#endif /* __cplusplus */
+void       gdict_speller_reset       (GDictSpeller *speller, dict_context_t *context);
+void       gdict_speller_reset_strat (GDictWindow *gdict, GDictSpeller *speller);
+void       gdict_speller_change_strategy (GDictSpeller *speller, gchar *new_strategy);
+void       gdict_speller_change_db (GDictSpeller *speller, gchar *new_db);
 
 #endif /* __GDICT_SPELLER_H_ */
