@@ -1,6 +1,3 @@
-#ifndef GNOMELOCALEDIR
-#define GNOMELOCALEDIR "/usr/share/locale"
-#endif
 /*  ----------------------------------------------------------------------
 
     Copyright (C) 1998  Cesar Miquel  (miquel@df.uba.ar)
@@ -21,10 +18,14 @@
 
     ---------------------------------------------------------------------- */
 
-
 #include <config.h>
 #include <gconf/gconf-client.h>
-#include <gnome.h>
+#include <libgnomevfs/gnome-vfs.h>
+#include <libgnomeui/gnome-ui-init.h>
+#include <libgnomeui/gnome-client.h>
+#include <gtk/gtk.h>
+#include <glib/gi18n.h>
+#include <glib/goption.h>
 #include "logview.h"
 #include "log_repaint.h"
 #include "logrtns.h"
@@ -35,12 +36,15 @@
 #include "desc_db.h"
 #include "misc.h"
 #include "logview-findbar.h"
-#include <popt.h>
+#include "userprefs.h"
+#include "configdata.h"
 
 static GObjectClass *parent_class;
 static GSList *logview_windows = NULL;
-static gchar *program_name = NULL;
 static GConfClient *client = NULL;
+static UserPrefsStruct *user_prefs;
+
+#define APP_NAME _("System Log Viewer")
 
 /*
  *    -------------------
@@ -49,16 +53,13 @@ static GConfClient *client = NULL;
  */
 
 void destroy (GObject *object, gpointer data);
-void ExitProg (GtkAction *action, GtkWidget *data);
-
 static void CreateMainWin (LogviewWindow *window_new);
-static void LoadLogMenu (GtkAction *action, GtkWidget *callback_data);
+static void LoadLogMenu (GtkAction *action, GtkWidget *parent_window);
 static void CloseLogMenu (GtkAction *action, GtkWidget *callback_data);
 static void FileSelectResponse (GtkWidget * chooser, gint response, gpointer data);
 static void open_databases (void);
+GtkWidget *logview_create_monitor_widget (LogviewWindow *window);
 static GtkWidget *logview_create_window (void);
-static void SaveUserPrefs(UserPrefsStruct *prefs);
-static char * parse_syslog(gchar * syslog_file);
 static void logview_menu_item_set_state (LogviewWindow *logviewwindow, char *path, gboolean state);
 static void toggle_calendar (GtkAction *action, GtkWidget *callback_data);
 static void toggle_zoom (GtkAction *action, GtkWidget *callback_data);
@@ -66,8 +67,13 @@ static void toggle_collapse_rows (GtkAction *action, GtkWidget *callback_data);
 static void toggle_monitor (GtkAction *action, GtkWidget *callback_data);
 static void logview_menus_set_state (LogviewWindow *logviewwindow);
 static void logview_search (GtkAction *action, GtkWidget *callback_data);
-static void logview_help (GtkAction *action, GtkWidget *callback_data);
+static void logview_help (GtkAction *action, GtkWidget *parent_window);
+static void logview_copy (GtkAction *action, GtkWidget *callback_data);
+static void logview_select_all (GtkAction *action, GtkWidget *callback_data);
 static int logview_count_logs (void);
+gboolean window_size_changed_cb (GtkWidget *widget, GdkEventConfigure *event, 
+				 gpointer data);
+GType logview_window_get_type (void);
 
 /*
  *    ,-------.
@@ -88,12 +94,16 @@ static GtkActionEntry entries[] = {
 	{ "CloseLog", GTK_STOCK_CLOSE, N_("Close"), "<control>W", N_("Close this log"), 
 	  G_CALLBACK (CloseLogMenu) },
 	{ "Quit", GTK_STOCK_QUIT, N_("_Quit"), "<control>Q", N_("Quit the log viewer"), 
-	  G_CALLBACK (ExitProg) },
+	  G_CALLBACK (gtk_main_quit) },
 
-	{ "Search", GTK_STOCK_FIND, N_("_Find"), "<control>F", N_("Find pattern in logs"),
+	{ "Copy", GTK_STOCK_COPY, N_("Copy"), "<control>C", N_("Copy the selection"),
+	  G_CALLBACK (logview_copy) },
+	{ "SelectAll", NULL, N_("Select All"), "<Control>A", N_("Select the entire log"),
+	  G_CALLBACK (logview_select_all) },
+	{ "Search", GTK_STOCK_FIND, N_("_Find..."), "<control>F", N_("Find pattern in logs"),
 	  G_CALLBACK (logview_search) },
 
-	{"CollapseAll", NULL, N_("Collapse _All"), "<control>A", N_("Collapse all the rows"),
+	{"CollapseAll", NULL, N_("Collapse _All"), NULL, N_("Collapse all the rows"),
 	 G_CALLBACK (toggle_collapse_rows) },
 
        	{ "HelpContents", GTK_STOCK_HELP, N_("_Contents"), "F1", N_("Open the help contents for the log viewer"), 
@@ -106,7 +116,7 @@ static GtkActionEntry entries[] = {
 static GtkToggleActionEntry toggle_entries[] = {
 	{ "MonitorLogs", NULL, N_("_Monitor"), "<control>M", N_("Monitor Current Log"),
 	  G_CALLBACK (toggle_monitor) },
-	{"ShowCalendar", NULL,  N_("_Calendar"), "<control>C", N_("Show Calendar Log"), 
+	{"ShowCalendar", NULL,  N_("Ca_lendar"), "<control>L", N_("Show Calendar Log"), 
 	 G_CALLBACK (toggle_calendar) },
 	{"ShowDetails", NULL,  N_("_Entry Detail"), "<control>E", N_("Show Entry Detail"), 
 	 G_CALLBACK (toggle_zoom) },
@@ -126,11 +136,15 @@ static const char *ui_description =
 	"			<menuitem action='Quit'/>"
 	"		</menu>"
 	"		<menu action='EditMenu'>"
+	"                       <menuitem action='Copy'/>"
+	"                       <menuitem action='SelectAll'/>"
+	"                       <separator/>"
 	"			<menuitem action='Search'/>"
 	"		</menu>"	
 	"		<menu action='ViewMenu'>"
 	"			<menuitem action='ShowCalendar'/>"
 	"			<menuitem action='ShowDetails'/>"
+	"                       <separator/>"
 	"			<menuitem action='CollapseAll'/>"
 	"		</menu>"
 	"		<menu action='HelpMenu'>"
@@ -147,26 +161,12 @@ static const char *ui_description =
  */
 	
 GList *regexp_db = NULL, *descript_db = NULL, *actions_db = NULL;
-UserPrefsStruct *user_prefs = NULL;
-UserPrefsStruct user_prefs_struct = {0};
 ConfigData *cfg = NULL;
-GtkWidget *open_file_dialog = NULL;
 gchar *file_to_open;
 
-poptContext poptCon;
-gint next_opt;
-
-struct poptOption options[] = {
-	{
-		"file",
-		'f',
-		POPT_ARG_STRING,
-		&(file_to_open),
-		1,
-		N_("Open the specified log."),
-		NULL
-	},
-	POPT_TABLEEND
+GOptionContext *context;
+GOptionEntry options[] = {
+	{ NULL }
 };
 
 /* ----------------------------------------------------------------------
@@ -181,8 +181,14 @@ destroy (GObject *object, gpointer data)
    logview_windows = g_slist_remove (logview_windows, window);
    if (window->monitored)
 	   monitor_stop (window);
-   if (logview_windows == NULL)
+   if (logview_windows == NULL) {
+	   if (window->curlog && !(window->curlog->display_name))
+		   user_prefs->logfile = window->curlog->name;
+	   else
+		   user_prefs->logfile = NULL;
+	   prefs_save (client, user_prefs);
 	   gtk_main_quit ();
+   }
 }
 
 static gint
@@ -198,12 +204,12 @@ save_session (GnomeClient *gnome_client, gint phase,
    numlogs = logview_count_logs ();
 
    argv = g_malloc0 (sizeof (gchar *) * numlogs+1);
-   argv[0] = program_name;
+   argv[0] = g_get_prgname();
 
    for (list = logview_windows; list != NULL; list = g_slist_next (list)) {
 	   LogviewWindow *w = list->data;
 	   if (w->curlog) {
-		   argv[i++] = g_strdup_printf ("--file=%s ", w->curlog->name);
+		   argv[i++] = g_strdup_printf ("%s", w->curlog->name);
 	   }
    }
    
@@ -236,15 +242,24 @@ logview_create_window_open_file (gchar *file)
 			logview->curlog = log;
 			logview->monitored = FALSE;
 			logview_menus_set_state (logview);
-			gtk_widget_show (window);
 			log_repaint (logview);
-			UpdateStatusArea(logview);
-			logview_set_window_title (logview);
 			gtk_widget_show (window);
 		}
 	}
 }
 
+GOptionContext *
+logview_init_options ()
+{
+	GOptionContext *context;
+	
+	context = g_option_context_new (" - Browse and monitor logs");
+	g_option_context_add_main_entries (context, options, NULL);
+	g_option_context_set_help_enabled (context, TRUE);
+	g_option_context_set_ignore_unknown_options (context, TRUE);
+	return context;
+}
+	
 /* ----------------------------------------------------------------------
    NAME:          main
    DESCRIPTION:   Program entry point.
@@ -254,7 +269,9 @@ int
 main (int argc, char *argv[])
 {
    GnomeClient *gnome_client;
+   GError *error;
    GnomeProgram *program;
+   int i;
 
    bindtextdomain(GETTEXT_PACKAGE, GNOMELOCALEDIR);
    bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
@@ -269,36 +286,30 @@ main (int argc, char *argv[])
 
    gconf_init (argc, argv, NULL);
    client = gconf_client_get_default ();
+   gnome_vfs_init ();
 
    gtk_window_set_default_icon_name ("logviewer");
-   /*  Load graphics config and prefs */
    cfg = CreateConfig();
    open_databases ();
-   user_prefs = &user_prefs_struct;
-   SetDefaultUserPrefs(user_prefs, client);
+   user_prefs = prefs_load (client);
    
-   program_name = (gchar *) argv[0];
-   poptCon = poptGetContext ("gnome-system-log", argc, (const gchar **) argv, 
-   							 options, 0);  
-   /* Open a new window for each log passed as a parameter */
-   while ((next_opt = poptGetNextOpt (poptCon)) > 0) {
-	   if ( next_opt == 1 ) {
-		   if (file_to_open) {
-			   logview_create_window_open_file (file_to_open);
-			   g_free (file_to_open);
-		   }
-	   }
-   }
+   context = logview_init_options ();
+   g_option_context_parse (context, &argc, &argv, &error);
 
+   /* Open a new window for each log passed as a parameter */
    /* If no log was passed as parameter, open regular logs */
-   if (logview_count_logs() == 0) {
-	   logview_create_window_open_file (user_prefs->logfile);
-	   if (logview_count_logs() == 0) {
-		   GtkWidget *window;
-		   ShowErrMessage (_("No log files to open"));
-		   window = logview_create_window ();
-		   gtk_widget_show (window);
+   if (argc > 1) {
+	   for (i=1; i<argc; i++) {
+		   file_to_open = argv[i];
+		   logview_create_window_open_file (file_to_open);
 	   }
+   } else
+	   logview_create_window_open_file (user_prefs->logfile);
+
+   if (logview_count_logs() == 0) {
+	   GtkWidget *window;
+	   window = logview_create_window ();
+	   gtk_widget_show (window);
    }
 
    gnome_client = gnome_master_client ();
@@ -338,14 +349,16 @@ logview_create_window ()
    logviewwindow->zoom_visible = FALSE;
    logviewwindow->zoom_scrolled_window = NULL;
    logviewwindow->zoom_dialog = NULL;
+   logviewwindow->clipboard = gtk_clipboard_get_for_display (gtk_widget_get_display (window),
+							     GDK_SELECTION_CLIPBOARD);
 
    /* FIXME : we need to listen to this key, not just read it. */
    gtk_ui_manager_set_add_tearoffs (logviewwindow->ui_manager, 
 				    gconf_client_get_bool 
 				    (client, "/desktop/gnome/interface/menus_have_tearoff", NULL));
 
-   gtk_signal_connect (GTK_OBJECT (window), "destroy",
-		       GTK_SIGNAL_FUNC (destroy), logviewwindow);
+   g_signal_connect (GTK_OBJECT (window), "destroy",
+		     G_CALLBACK (destroy), logviewwindow);
 
    logview_windows = g_slist_prepend (logview_windows, window);
 
@@ -365,18 +378,15 @@ CreateMainWin (LogviewWindow *window)
    GtkTreeSelection *selection;
    GtkTreeViewColumn *column;
    GtkCellRenderer *renderer;
-   GtkWidget *scrolled_window = NULL;
-   gint i, f;
+   gint i;
    GtkActionGroup *action_group;
    GtkAccelGroup *accel_group;
-   GError *error;
+   GError *error = NULL;
    GtkWidget *menubar;
-   PangoContext      *context;
-   PangoFontMetrics  *metrics;
    const gchar *column_titles[] = { N_("Date"), N_("Host Name"),
                                     N_("Process"), N_("Message"), NULL };
 
-   gtk_window_set_default_size (GTK_WINDOW (window), LOG_CANVAS_W, LOG_CANVAS_H);
+   gtk_window_set_default_size (GTK_WINDOW (window), user_prefs->width, user_prefs->height);
 
    vbox = gtk_vbox_new (FALSE, 0);
    gtk_container_add (GTK_CONTAINER (window), vbox);
@@ -394,11 +404,10 @@ CreateMainWin (LogviewWindow *window)
    accel_group = gtk_ui_manager_get_accel_group (window->ui_manager);
    gtk_window_add_accel_group (GTK_WINDOW (window), accel_group);
    
-   error = NULL;
    if (!gtk_ui_manager_add_ui_from_string (window->ui_manager, ui_description, -1, &error)) {
 	   g_message ("Building menus failed: %s", error->message);
 	   g_error_free (error);
-	   exit (EXIT_FAILURE);
+	   exit (1);
    }
    
    menubar = gtk_ui_manager_get_widget (window->ui_manager, "/LogviewMenu");
@@ -407,14 +416,12 @@ CreateMainWin (LogviewWindow *window)
    window->main_view = gtk_frame_new (NULL);
    gtk_box_pack_start (GTK_BOX (vbox), window->main_view, TRUE, TRUE, 0);
 
-   /* Create scrolled window and tree view */
+   /* Scrolled windows for the main view and monitor view */
    window->log_scrolled_window = gtk_scrolled_window_new (NULL, NULL);
-   gtk_widget_set_sensitive (window->log_scrolled_window, TRUE);
    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (window->log_scrolled_window),
                GTK_POLICY_AUTOMATIC,
                GTK_POLICY_AUTOMATIC);
-
-   window->mon_scrolled_window = monitor_create_widget (window);
+   window->mon_scrolled_window = logview_create_monitor_widget (window);
 
    /* We ref the two scrolled_windows so they are not destroyed when we remove one from
     * the main_view to put the other */
@@ -423,24 +430,20 @@ CreateMainWin (LogviewWindow *window)
 
    gtk_container_add (GTK_CONTAINER (window->main_view), window->log_scrolled_window);
 
-   /* Create Tree View */
-   tree_store = gtk_tree_store_new (4,
-                G_TYPE_STRING, G_TYPE_STRING,
-                G_TYPE_STRING, G_TYPE_STRING);
+   /* Main Tree View */
+   tree_store = gtk_tree_store_new (4, G_TYPE_STRING, G_TYPE_STRING,
+				    G_TYPE_STRING, G_TYPE_STRING);
 
    window->view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (tree_store));
    gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (window->view), TRUE);
    g_object_unref (G_OBJECT (tree_store)); 
    
-   /* Add Tree View Columns */
-
    for (i = 0; column_titles[i]; i++) {
         renderer = gtk_cell_renderer_text_new ();
         column = gtk_tree_view_column_new_with_attributes (_(column_titles[i]),
                     renderer, "text", i, NULL);
         gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_AUTOSIZE); 
         gtk_tree_view_column_set_resizable (column, TRUE);
-        gtk_tree_view_column_set_spacing (column, 6);
         gtk_tree_view_append_column (GTK_TREE_VIEW (window->view), column);
    }
 
@@ -448,14 +451,15 @@ CreateMainWin (LogviewWindow *window)
    gtk_widget_show_all (window->log_scrolled_window);
 
    selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (window->view));
-   gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);
+   gtk_tree_selection_set_mode (selection, GTK_SELECTION_MULTIPLE);
 
    /* Add signal handlers */
    g_signal_connect (G_OBJECT (selection), "changed",
                      G_CALLBACK (handle_selection_changed_cb), window);
-
    g_signal_connect (G_OBJECT (window->view), "row_activated",
                      G_CALLBACK (handle_row_activation_cb), window);
+   g_signal_connect (G_OBJECT (window), "configure_event",
+		     G_CALLBACK (window_size_changed_cb), window);
 
    window->find_bar = gtk_toolbar_new ();
    logview_findbar_populate (window, window->find_bar);
@@ -463,7 +467,7 @@ CreateMainWin (LogviewWindow *window)
    gtk_box_pack_start (GTK_BOX (vbox), window->find_bar, FALSE, FALSE, 0);
    gtk_widget_show (window->find_bar);
 
-   /* Create status area at bottom */
+   /* Status area at bottom */
    window->statusbar = gtk_statusbar_new ();
    gtk_box_pack_start (GTK_BOX (vbox), window->statusbar, FALSE, FALSE, 0);
 
@@ -483,19 +487,28 @@ CloseLogMenu (GtkAction *action, GtkWidget *callback_data)
 
    g_return_if_fail (window->curlog);
 
-   CloseLog (window->curlog);
+   if (window->monitored) {
+	   GtkAction *action = gtk_ui_manager_get_action (window->ui_manager, "/LogviewMenu/FileMenu/MonitorLogs");
+	   gtk_action_activate (action);
+   }
+
+   if (window->calendar_visible) {
+	   GtkAction *action = gtk_ui_manager_get_action (window->ui_manager, "/LogviewMenu/ViewMenu/ShowCalendar");
+	   gtk_action_activate (action);
+   }
+
+   if (window->zoom_visible) {
+	   GtkAction *action = gtk_ui_manager_get_action (window->ui_manager, "/LogviewMenu/ViewMenu/ShowDetails");
+	   gtk_action_activate (action);
+   }
 
    gtk_widget_hide (window->find_bar);
 
+   CloseLog (window->curlog);
    window->curlog = NULL;
    logview_menus_set_state (window);
+   logview_set_window_title (window);
    log_repaint (window);
-   if (window->loginfovisible)
-	   RepaintLogInfo (window);
-   if (window->monitored) {
-	   monitor_stop (window);
-	   window->monitored = FALSE;
-   }
 }
 
 /* ----------------------------------------------------------------------
@@ -507,17 +520,13 @@ static void
 FileSelectResponse (GtkWidget * chooser, gint response, gpointer data)
 {
    char *f;
-   Log *tl;
-   gint i;
    LogviewWindow *window = data;
 
-   if (response != GTK_RESPONSE_OK) {
-	   gtk_widget_destroy (chooser);
+   gtk_widget_hide (GTK_WIDGET (chooser));
+   if (response != GTK_RESPONSE_OK)
 	   return;
-   }
    
    f = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (chooser));
-   gtk_widget_destroy (GTK_WIDGET (chooser));
 
    if (f != NULL) {
 	   /* Check if the log is not already opened */
@@ -537,6 +546,7 @@ FileSelectResponse (GtkWidget * chooser, gint response, gpointer data)
 	   if (window->curlog) {
 		   logview_create_window_open_file (f);		   
 	   } else {
+		   Log *tl;
 		   if ((tl = OpenLogFile (f)) != NULL) {
 			   window->curlog = tl;
 			   window->curlog->first_time = TRUE; 
@@ -544,12 +554,9 @@ FileSelectResponse (GtkWidget * chooser, gint response, gpointer data)
 			   
 			   log_repaint (window);
 			   
-			   if (window->loginfovisible)
-				   RepaintLogInfo (window);
 			   if (window->calendar_visible)
 				   init_calendar_data(window);
 			   
-			   UpdateStatusArea(window);
 			   logview_menus_set_state (window);
 		   }
 	   }
@@ -565,61 +572,60 @@ FileSelectResponse (GtkWidget * chooser, gint response, gpointer data)
    ---------------------------------------------------------------------- */
 
 static void
-LoadLogMenu (GtkAction *action, GtkWidget *callback_data)
+LoadLogMenu (GtkAction *action, GtkWidget *parent_window)
 {
-   GtkWidget *chooser = NULL;
-   LogviewWindow *window = LOGVIEW_WINDOW (callback_data);
+   static GtkWidget *chooser = NULL;
    
-   /*  Cannot have more than one file chooser window */
-   /*  at one time. */
-   if (open_file_dialog != NULL) {
-	   gtk_widget_show_now (open_file_dialog);
-	   gdk_window_raise (open_file_dialog->window);
-	   return;
+   if (chooser == NULL) {
+	   chooser = gtk_file_chooser_dialog_new (_("Open new logfile"),
+						  GTK_WINDOW (parent_window),
+						  GTK_FILE_CHOOSER_ACTION_OPEN,
+						  GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+						  GTK_STOCK_OPEN, GTK_RESPONSE_OK,
+						  NULL);
+	   gtk_dialog_set_default_response (GTK_DIALOG (chooser), GTK_RESPONSE_OK);
+	   gtk_window_set_modal (GTK_WINDOW (chooser), TRUE);
+	   g_signal_connect (G_OBJECT (chooser), "response",
+			     G_CALLBACK (FileSelectResponse), parent_window);
+	   g_signal_connect (G_OBJECT (chooser), "destroy",
+			     G_CALLBACK (gtk_widget_destroyed), &chooser);
+	   if (user_prefs->logfile != NULL)
+		   gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (chooser), 
+						  user_prefs->logfile);
    }
 
-   chooser = gtk_file_chooser_dialog_new (_("Open new logfile"),
-					  GTK_WINDOW (window),
-					  GTK_FILE_CHOOSER_ACTION_OPEN,
-					  GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-					  GTK_STOCK_OPEN, GTK_RESPONSE_OK,
-					  NULL);
-   gtk_dialog_set_default_response (GTK_DIALOG (chooser),
-		   		    GTK_RESPONSE_OK);
-   gtk_window_set_default_size (GTK_WINDOW (chooser), 600, 400);
-
-   /* Make window modal */
-   gtk_window_set_modal (GTK_WINDOW (chooser), TRUE);
-
-   if (user_prefs->logfile != NULL)
-   	gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (chooser), 
-				       user_prefs->logfile);
-
-   gtk_window_set_position (GTK_WINDOW (chooser), GTK_WIN_POS_MOUSE);
-   g_signal_connect (G_OBJECT (chooser), "response",
-		     G_CALLBACK (FileSelectResponse), window);
-
-   g_signal_connect (G_OBJECT (chooser), "destroy",
-		     G_CALLBACK (gtk_widget_destroyed), &open_file_dialog);
-
-   gtk_widget_show (chooser);
-
-   open_file_dialog = chooser;
-
+   gtk_window_present (GTK_WINDOW (chooser));
 }
 
-/* ----------------------------------------------------------------------
-   NAME:          ExitProg
-   DESCRIPTION:   Callback to call when program exits.
-   ---------------------------------------------------------------------- */
-
-void 
-ExitProg (GtkAction *action, GtkWidget *data)
+GtkWidget *logview_create_monitor_widget (LogviewWindow *window)
 {
-   LogviewWindow *window = LOGVIEW_WINDOW (data);
+	GtkWidget *clist_view, *swin;
+	GtkListStore *clist;   
+	GtkCellRenderer *clist_cellrenderer;
+	GtkTreeViewColumn *clist_column;
 
-   gtk_main_quit ();
+	clist = gtk_list_store_new (1, G_TYPE_STRING);
+	clist_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (clist));
+	g_object_unref (clist);
+	clist_cellrenderer = gtk_cell_renderer_text_new ();
+	clist_column = gtk_tree_view_column_new_with_attributes
+		(NULL, clist_cellrenderer, "markup", 0, NULL);
+	gtk_tree_view_append_column (GTK_TREE_VIEW (clist_view), clist_column);
+	gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (clist_view), FALSE);
+	swin = gtk_scrolled_window_new (NULL, NULL);
+	gtk_container_add (GTK_CONTAINER (swin), GTK_WIDGET (clist_view));
+	gtk_tree_selection_set_mode
+		( (GtkTreeSelection *)gtk_tree_view_get_selection
+		  (GTK_TREE_VIEW (clist_view)),
+		  GTK_SELECTION_MULTIPLE);
+	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (swin),
+					GTK_POLICY_AUTOMATIC,
+					GTK_POLICY_AUTOMATIC);
+	gtk_widget_show_all (swin);
 
+	window->mon_list_view = (GtkWidget *) clist_view;
+
+	return swin;
 }
 
 /* ----------------------------------------------------------------------
@@ -711,110 +717,6 @@ open_databases (void)
 
 }
 
-/* ----------------------------------------------------------------------
-   NAME:          IsLeapYear
-   DESCRIPTION:   Return TRUE if year is a leap year.
-   ---------------------------------------------------------------------- */
-int
-IsLeapYear (int year)
-{
-   if ((1900 + year) % 4 == 0)
-      return TRUE;
-   else
-      return FALSE;
-
-}
-
-void SetDefaultUserPrefs(UserPrefsStruct *prefs, GConfClient *client)
-{
-	/* Make defaults configurable later */
-	/* Will have to save prefs. eventually too*/
-	gchar *logfile = NULL;
-	struct stat filestat;
-	
-	logfile = gconf_client_get_string (client, "/apps/gnome-system-log/logfile", NULL);
-	if (logfile != NULL && strcmp (logfile, "")) {
-		prefs->logfile = g_strdup (logfile);
-		g_free (logfile);
-	}
-	else {
-
-		/* For first time running, try parsing /etc/syslog.conf */
-		if (lstat("/etc/syslog.conf", &filestat) == 0) {
-			if ((logfile = parse_syslog("/etc/syslog.conf")) == NULL);
-			prefs->logfile = g_strdup (logfile);
-		}
-		else if (lstat("/var/adm/messages", &filestat) == 0) 
-			prefs->logfile = g_strdup ("/var/adm/messages");
-		else if (lstat("/var/log/messages", &filestat) == 0) 
-			prefs->logfile = g_strdup ("/var/log/messages");
-		else if (lstat("/var/log/sys.log", &filestat) == 0) 
-			prefs->logfile = g_strdup ("/var/log/sys.log");
-		else
-			prefs->logfile = NULL;
-	}
-}
-
-char * parse_syslog(gchar * syslog_file) {
-/* Most of this stolen from sysklogd sources */
-    char * logfile = NULL;
-    char cbuf[BUFSIZ];
-    char *cline;
-    char *p;
-    FILE * cf;
-    if ((cf = fopen(syslog_file, "r")) == NULL) {
-        fprintf(stderr, "Could not open file: (%s)\n", syslog_file);
-        return NULL;
-    }
-    cline = cbuf;
-    while (fgets(cline, sizeof(cbuf) - (cline - cbuf), cf) != NULL) {
-        for (p = cline; isspace(*p); ++p);
-        if (*p == '\0' || *p == '#')
-            continue;
-        for (;*p && !strchr("\t ", *p); ++p);
-        while (*p == '\t' || *p == ' ')
-            p++;
-        if (*p == '-')
-            p++;
-        if (*p == '/') {
-            logfile = g_strdup (p);
-            /* remove trailing newline */
-            if (logfile[strlen(logfile)-1] == '\n')
-                logfile[strlen(logfile)-1] = '\0';
-            fprintf(stderr, "Found a logfile: (%s)\n", logfile);
-            return logfile;
-        }
-        /* I don't totally understand this code
-           it came from syslogd.c and is supposed
-           to read run-on lines that end with \
-           FIXME?? */
-        /*
-        if (*p == '\\') {
-            if ((p - cbuf) > BUFSIZ - 30) {
-                cline = cbuf;
-            } else {
-                *p = 0;
-                cline = p;
-                continue;
-            }
-        }  else
-            cline = cbuf;
-        *++p = '\0';
-        */
-        
-    }
-    return logfile; 
-}
-
-
- 
-void SaveUserPrefs(UserPrefsStruct *prefs)
-{
-    if (gconf_client_key_is_writable (client, "/apps/gnome-system-log/logfile", NULL) &&
-	prefs->logfile != NULL)
-	    gconf_client_set_string (client, "/apps/gnome-system-log/logfile", prefs->logfile, NULL);
-}
-
 static void 
 toggle_calendar (GtkAction *action, GtkWidget *callback_data)
 {
@@ -832,11 +734,10 @@ toggle_zoom (GtkAction *action, GtkWidget *callback_data)
 {
     LogviewWindow *window = LOGVIEW_WINDOW (callback_data);
     if (window->zoom_visible) {
-	    close_zoom_view (window);
-    } else  {
-	create_zoom_view (window);
-    }
-
+	    window->zoom_visible = FALSE;
+	    gtk_widget_hide (window->zoom_dialog);
+    } else
+	    create_zoom_view (window);
 }
 
 static void 
@@ -850,49 +751,105 @@ static void
 toggle_monitor (GtkAction *action, GtkWidget *callback_data)
 {
     LogviewWindow *window = LOGVIEW_WINDOW (callback_data);
-    if (window->monitored) {
-	    gtk_container_remove (GTK_CONTAINER (window->main_view), window->mon_scrolled_window);
-	    monitor_stop (window);
-	    gtk_container_add (GTK_CONTAINER (window->main_view), window->log_scrolled_window);
-	    window->monitored = FALSE;
-    } else {
-	    gtk_container_remove (GTK_CONTAINER(window->main_view), window->log_scrolled_window);
-	    mon_read_last_page (window);
-	    gtk_container_add (GTK_CONTAINER(window->main_view), window->mon_scrolled_window);
-	    go_monitor_log (window);
-	    window->monitored = TRUE;
+    g_return_if_fail (window->curlog);
+    if (!window->curlog->display_name) {
+	    if (window->monitored) {
+		    gtk_container_remove (GTK_CONTAINER (window->main_view), window->mon_scrolled_window);
+		    gtk_container_add (GTK_CONTAINER (window->main_view), window->log_scrolled_window);
+		    monitor_stop (window);
+		    window->monitored = FALSE;
+	    } else {
+		    gtk_container_remove (GTK_CONTAINER(window->main_view), window->log_scrolled_window);
+		    gtk_container_add (GTK_CONTAINER(window->main_view), window->mon_scrolled_window);
+		    monitor_start (window);
+		    window->monitored = TRUE;
+	    }
+	    logview_set_window_title (window);
+	    logview_menus_set_state (window);
     }
-    logview_set_window_title (window);
 }
 
 static void
 logview_search (GtkAction *action,GtkWidget *callback_data)
 {
-	static GtkWidget *dialog = NULL;
 	LogviewWindow *window = LOGVIEW_WINDOW (callback_data);
 
 	gtk_widget_show (window->find_bar);
 	gtk_widget_grab_focus (window->find_entry);
 }
 
+static void
+logview_copy (GtkAction *action, GtkWidget *callback_data)
+{
+	LogviewWindow *window = LOGVIEW_WINDOW (callback_data);
+	LogLine *line;
+	int nline, i;
+	gchar *text, **lines;
+
+	if (window->curlog->selected_line_first > -1 && window->curlog->selected_line_last > -1) {
+		nline = window->curlog->selected_line_last - window->curlog->selected_line_first + 1;
+		lines = g_new0(gchar *, (nline + 1));
+		for (i=0; i<=nline; i++) {
+			if (window->curlog->selected_line_first + i < window->curlog->total_lines) {
+				line = (window->curlog->lines)[window->curlog->selected_line_first + i];
+				if (line && line->hostname && line->process && line->message)
+					lines[i] = g_strdup_printf ("%s %s %s", line->hostname, line->process, line->message);
+			}
+		}
+		lines[nline] = NULL;
+		text = g_strjoinv ("\n", lines);
+		gtk_clipboard_set_text (window->clipboard, LocaleToUTF8(text), -1);
+		g_free (text);
+		g_strfreev (lines);
+	}
+}
+
+static void
+logview_select_all (GtkAction *action, GtkWidget *callback_data)
+{
+	GtkTreeSelection *selection;
+	LogviewWindow *window = LOGVIEW_WINDOW (callback_data);
+	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (window->view));
+	gtk_tree_selection_select_all (selection);
+}
+
 static void
 logview_menu_item_set_state (LogviewWindow *logviewwindow, char *path, gboolean state)
 {
+	GtkAction *action;
 	g_return_if_fail (path);
-
-	gtk_widget_set_sensitive (GTK_WIDGET (gtk_ui_manager_get_widget(logviewwindow->ui_manager, path)), state);
+	action = gtk_ui_manager_get_action (logviewwindow->ui_manager, path);
+	gtk_action_set_sensitive (action, state);
 }
 
 static void
 logview_menus_set_state (LogviewWindow *window)
 {
-	logview_menu_item_set_state (window, "/LogviewMenu/FileMenu/Properties", (window->curlog != NULL));
-	logview_menu_item_set_state (window, "/LogviewMenu/FileMenu/CloseLog", (window->curlog != NULL));
-	logview_menu_item_set_state (window, "/LogviewMenu/FileMenu/MonitorLogs", (window->curlog != NULL));
-	logview_menu_item_set_state (window, "/LogviewMenu/ViewMenu/ShowCalendar", (window->curlog != NULL));
-	logview_menu_item_set_state (window, "/LogviewMenu/ViewMenu/ShowDetails", (window->curlog != NULL));
-	logview_menu_item_set_state (window, "/LogviewMenu/ViewMenu/CollapseAll", (window->curlog != NULL));
-	logview_menu_item_set_state (window, "/LogviewMenu/EditMenu/Search", (window->curlog != NULL));
+	if (window->monitored) {
+		logview_menu_item_set_state (window, "/LogviewMenu/EditMenu/Copy", FALSE);
+		logview_menu_item_set_state (window, "/LogviewMenu/EditMenu/Search", FALSE);
+		logview_menu_item_set_state (window, "/LogviewMenu/ViewMenu/ShowCalendar", FALSE);
+		logview_menu_item_set_state (window, "/LogviewMenu/ViewMenu/ShowDetails", FALSE); 
+		logview_menu_item_set_state (window, "/LogviewMenu/ViewMenu/CollapseAll", FALSE);
+		logview_menu_item_set_state (window, "/LogviewMenu/EditMenu/SelectAll", FALSE);
+	} else {
+		if (window->curlog) {
+			if (window->curlog->display_name)
+				logview_menu_item_set_state (window, "/LogviewMenu/FileMenu/MonitorLogs", FALSE);
+			else
+				logview_menu_item_set_state (window, "/LogviewMenu/FileMenu/MonitorLogs", TRUE);
+		} else
+			logview_menu_item_set_state (window, "/LogviewMenu/FileMenu/MonitorLogs", FALSE);
+		
+		logview_menu_item_set_state (window, "/LogviewMenu/FileMenu/Properties", (window->curlog != NULL));
+		logview_menu_item_set_state (window, "/LogviewMenu/FileMenu/CloseLog", (window->curlog != NULL));
+		logview_menu_item_set_state (window, "/LogviewMenu/ViewMenu/ShowCalendar", (window->curlog != NULL));
+		logview_menu_item_set_state (window, "/LogviewMenu/ViewMenu/ShowDetails", (window->curlog != NULL));
+		logview_menu_item_set_state (window, "/LogviewMenu/ViewMenu/CollapseAll", (window->curlog != NULL));
+		logview_menu_item_set_state (window, "/LogviewMenu/EditMenu/Search", (window->curlog != NULL));
+		logview_menu_item_set_state (window, "/LogviewMenu/EditMenu/Copy", (window->curlog != NULL));
+		logview_menu_item_set_state (window, "/LogviewMenu/EditMenu/SelectAll", (window->curlog != NULL));
+	}
 }
 
 static int
@@ -910,17 +867,13 @@ logview_count_logs (void)
 }
 
 static void
-logview_help (GtkAction *action, GtkWidget *callback_data)
+logview_help (GtkAction *action, GtkWidget *parent_window)
 {
-        GError *error = NULL;
-	LogviewWindow *window = LOGVIEW_WINDOW(callback_data);
-                                                                                
+        GError *error = NULL;                                                                                
         gnome_help_display_desktop_on_screen (NULL, "gnome-system-log", "gnome-system-log", NULL,
-					      gtk_widget_get_screen (GTK_WIDGET(window)), &error);
+					      gtk_widget_get_screen (GTK_WIDGET(parent_window)), &error);
 	if (error) {
-		gchar *message;
-		message = g_strdup_printf (_("There was an error displaying help: %s"), error->message);
-		ShowErrMessage (message);
+		ShowErrMessage (GTK_WIDGET(parent_window), _("There was an error displaying help."), error->message);
 		g_error_free (error);
 	}
 }
@@ -929,11 +882,19 @@ void
 logview_set_window_title (LogviewWindow *window)
 {
 	gchar *window_title;
-	if ((window->curlog != NULL) && (window->curlog->name != NULL))
-		if (window->monitored) 
-			window_title = g_strdup_printf (_("%s - %s (monitored)"), window->curlog->name, APP_NAME);
+	gchar *logname;
+
+	if ((window->curlog != NULL) && (window->curlog->name != NULL)) {
+		if (window->curlog->display_name != NULL)
+			logname = window->curlog->display_name;
 		else
-			window_title = g_strdup_printf ("%s - %s", window->curlog->name, APP_NAME);
+			logname = window->curlog->name;
+		
+		if (window->monitored) 
+			window_title = g_strdup_printf (_("%s (monitored) - %s"), logname, APP_NAME);
+		else
+			window_title = g_strdup_printf ("%s - %s", logname, APP_NAME);
+	}
 	else
 		window_title = g_strdup_printf (APP_NAME);
 	gtk_window_set_title (GTK_WINDOW (window), window_title);
@@ -943,11 +904,9 @@ logview_set_window_title (LogviewWindow *window)
 static void
 logview_window_finalize (GObject *object)
 {
-	int i;
         LogviewWindow *window = (LogviewWindow *) object;
 
 	g_object_unref (window->ui_manager);
-	SaveUserPrefs(user_prefs);
         parent_class->finalize (object);
 }
 
@@ -985,3 +944,12 @@ logview_window_get_type (void)
 	return object_type;
 }
 
+gboolean 
+window_size_changed_cb (GtkWidget *widget, GdkEventConfigure *event, 
+				 gpointer data)
+{
+	LogviewWindow *window = data;
+
+	prefs_store_size (GTK_WIDGET(window), user_prefs);
+	return FALSE;
+}
